@@ -1,17 +1,22 @@
 /* apps/crl.c */
-/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1997 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
+ *
+ * This package is an SSL implementation written
+ * by Eric Young (eay@cryptsoft.com).
+ * The implementation was written so as to conform with Netscapes SSL.
  * 
- * This file is part of an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
- * The implementation was written so as to conform with Netscapes SSL
- * specification.  This library and applications are
- * FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
- * as long as the following conditions are aheared to.
+ * This library is free for commercial and non-commercial use as long as
+ * the following conditions are aheared to.  The following conditions
+ * apply to all code found in this distribution, be it the RC4, RSA,
+ * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
+ * included with this distribution is covered by the same copyright terms
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
- * the code are not to be removed.  If this code is used in a product,
- * Eric Young should be given attribution as the author of the parts used.
+ * the code are not to be removed.
+ * If this package is used in a product, Eric Young should be given attribution
+ * as the author of the parts of the library used.
  * This can be in the form of a textual message at program startup or
  * in documentation (online or textual) provided with the package.
  * 
@@ -25,7 +30,13 @@
  *    documentation and/or other materials provided with the distribution.
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
- *    This product includes software developed by Eric Young (eay@mincom.oz.au)
+ *    "This product includes cryptographic software written by
+ *     Eric Young (eay@cryptsoft.com)"
+ *    The word 'cryptographic' can be left out if the rouines from the library
+ *    being used are not cryptographic related :-).
+ * 4. If you include any Windows specific code (or a derivative thereof) from 
+ *    the apps directory (application code) you must include an acknowledgement:
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -49,7 +60,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include "apps.h"
-#include "buffer.h"
+#include "bio.h"
 #include "err.h"
 #include "x509.h"
 #include "pem.h"
@@ -65,11 +76,12 @@
 #define FORMAT_TEXT	2
 #define FORMAT_PEM	3
 
-static char *ctl_usage[]={
+static char *crl_usage[]={
 "usage: crl args\n",
 "\n",
 " -inform arg     - input format - default PEM (one of DER, TXT or PEM)\n",
 " -outform arg    - output format - default PEM\n",
+" -text           - print out a text format version\n",
 " -in arg         - input file - default stdin\n",
 " -out arg        - output file - default stdout\n",
 " -hash           - print hash value\n",
@@ -86,6 +98,8 @@ static X509_CRL *load_crl(char *file, int format);
 static X509_CRL *load_crl();
 #endif
 
+static BIO *bio_out=NULL;
+
 int MAIN(argc, argv)
 int argc;
 char **argv;
@@ -97,7 +111,7 @@ char **argv;
 	char *infile=NULL,*outfile=NULL;
 	char *str=NULL;
 	int hash=0,issuer=0,lastupdate=0,nextupdate=0,noout=0;
-	char **pp;
+	char **pp,buf[256];
 
 	apps_startup();
 
@@ -105,6 +119,10 @@ char **argv;
 		if ((bio_err=BIO_new(BIO_s_file())) != NULL)
 			BIO_set_fp(bio_err,stderr,BIO_NOCLOSE);
 
+	if (bio_out == NULL)
+		if ((bio_out=BIO_new(BIO_s_file())) != NULL)
+			BIO_set_fp(bio_out,stdout,BIO_NOCLOSE);
+
 	informat=FORMAT_PEM;
 	outformat=FORMAT_PEM;
 
@@ -130,6 +148,10 @@ char **argv;
 			if (--argc < 1) goto bad;
 			outformat=str2fmt(*(++argv));
 			}
+		else if (strcmp(*argv,"-text") == 0)
+			{
+			outformat=FORMAT_TEXT;
+			}
 		else if (strcmp(*argv,"-in") == 0)
 			{
 			if (--argc < 1) goto bad;
@@ -152,7 +174,7 @@ char **argv;
 			noout= ++num;
 		else
 			{
-			fprintf(stderr,"unknown option %s\n",*argv);
+			BIO_printf(bio_err,"unknown option %s\n",*argv);
 			badops=1;
 			break;
 			}
@@ -160,11 +182,19 @@ char **argv;
 		argv++;
 		}
 
+	if (outformat == FORMAT_TEXT)
+		{
+		num=0;
+		issuer= ++num;
+		lastupdate= ++num;
+		nextupdate= ++num;
+		}
+
 	if (badops)
 		{
 bad:
-		for (pp=ctl_usage; (*pp != NULL); pp++)
-			fprintf(stderr,*pp);
+		for (pp=crl_usage; (*pp != NULL); pp++)
+			BIO_printf(bio_err,*pp);
 		goto end;
 		}
 
@@ -178,31 +208,26 @@ bad:
 			{
 			if (issuer == i)
 				{
-				str=X509_NAME_oneline(x->crl->issuer);
-				if (str == NULL)
-					{
-					fprintf(stderr,"unable to get issuer Name from CRL\n");
-					ERR_print_errors(bio_err);
-					goto end;
-					}
+				X509_NAME_oneline(x->crl->issuer,buf,256);
 				fprintf(stdout,"issuer= %s\n",str);
-				free(str);
 				}
 
 			if (hash == i)
 				{
 				fprintf(stdout,"%08lx\n",
-					X509_name_hash(x->crl->issuer));
+					X509_NAME_hash(x->crl->issuer));
 				}
 			if (lastupdate == i)
 				{
-				fprintf(stdout,"lastUpdate=%s\n",
-					x->crl->lastUpdate);
+				fprintf(stdout,"lastUpdate=");
+				ASN1_UTCTIME_print(bio_out,x->crl->lastUpdate);
+				fprintf(stdout,"\n");
 				}
 			if (nextupdate == i)
 				{
-				fprintf(stdout,"nextUpdate=%s\n",
-					x->crl->nextUpdate);
+				fprintf(stdout,"nextUpdate=");
+				ASN1_UTCTIME_print(bio_out,x->crl->nextUpdate);
+				fprintf(stdout,"\n");
 				}
 			}
 		}
@@ -231,14 +256,30 @@ bad:
 		i=(int)i2d_X509_CRL_bio(out,x);
 	else if (outformat == FORMAT_PEM)
 		i=PEM_write_bio_X509_CRL(out,x);
-	else	{
-		fprintf(stderr,"bad output format specified for outfile\n");
+	else if (outformat == FORMAT_TEXT)
+		{
+		X509_REVOKED *r;
+
+		while ((r=(X509_REVOKED *)sk_pop(x->crl->revoked)) != NULL)
+			{
+			fprintf(stdout,"revoked: serialNumber=");
+			i2a_ASN1_INTEGER(out,r->serialNumber);
+			fprintf(stdout," revocationDate=");
+			ASN1_UTCTIME_print(bio_out,r->revocationDate);
+			fprintf(stdout,"\n");
+			}
+		i=1;
+		}
+	else	
+		{
+		BIO_printf(bio_err,"bad output format specified for outfile\n");
 		goto end;
 		}
-	if (!i) { fprintf(stderr,"unable to write CRL\n"); goto end; }
+	if (!i) { BIO_printf(bio_err,"unable to write CRL\n"); goto end; }
 	ret=0;
 end:
 	if (out != NULL) BIO_free(out);
+	if (bio_out != NULL) BIO_free(bio_out);
 	if (x != NULL) X509_CRL_free(x);
 	EXIT(ret);
 	}
@@ -272,12 +313,12 @@ int format;
 	else if (format == FORMAT_PEM)
 		x=PEM_read_bio_X509_CRL(in,NULL,NULL);
 	else	{
-		fprintf(stderr,"bad input format specified for input crl\n");
+		BIO_printf(bio_err,"bad input format specified for input crl\n");
 		goto end;
 		}
 	if (x == NULL)
 		{
-		fprintf(stderr,"unable to load CRL\n");
+		BIO_printf(bio_err,"unable to load CRL\n");
 		ERR_print_errors(bio_err);
 		goto end;
 		}
