@@ -1,17 +1,22 @@
 /* apps/crl2p7.c */
-/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1997 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
+ *
+ * This package is an SSL implementation written
+ * by Eric Young (eay@cryptsoft.com).
+ * The implementation was written so as to conform with Netscapes SSL.
  * 
- * This file is part of an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
- * The implementation was written so as to conform with Netscapes SSL
- * specification.  This library and applications are
- * FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
- * as long as the following conditions are aheared to.
+ * This library is free for commercial and non-commercial use as long as
+ * the following conditions are aheared to.  The following conditions
+ * apply to all code found in this distribution, be it the RC4, RSA,
+ * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
+ * included with this distribution is covered by the same copyright terms
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
- * the code are not to be removed.  If this code is used in a product,
- * Eric Young should be given attribution as the author of the parts used.
+ * the code are not to be removed.
+ * If this package is used in a product, Eric Young should be given attribution
+ * as the author of the parts of the library used.
  * This can be in the form of a textual message at program startup or
  * in documentation (online or textual) provided with the package.
  * 
@@ -25,7 +30,13 @@
  *    documentation and/or other materials provided with the distribution.
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
- *    This product includes software developed by Eric Young (eay@mincom.oz.au)
+ *    "This product includes cryptographic software written by
+ *     Eric Young (eay@cryptsoft.com)"
+ *    The word 'cryptographic' can be left out if the rouines from the library
+ *    being used are not cryptographic related :-).
+ * 4. If you include any Windows specific code (or a derivative thereof) from 
+ *    the apps directory (application code) you must include an acknowledgement:
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -44,6 +55,7 @@
  * copied and put under another distribution licence
  * [including the GNU Public Licence.]
  */
+
 /* This was written by Gordon Chaffee <chaffee@plateau.cs.berkeley.edu>
  * and donated 'to the cause' along with lots and lots of other fixes to
  * the library. */
@@ -54,7 +66,7 @@
 #include <sys/stat.h>
 #include "apps.h"
 #include "err.h"
-#include "envelope.h"
+#include "evp.h"
 #include "x509.h"
 #include "pkcs7.h"
 #include "pem.h"
@@ -88,7 +100,7 @@ char **argv;
 	X509_CRL *crl=NULL;
 	STACK *crl_stack=NULL;
 	STACK *cert_stack=NULL;
-	int ret=1;
+	int ret=1,nocrl=0;
 
 	apps_startup();
 
@@ -122,6 +134,10 @@ char **argv;
 			if (--argc < 1) goto bad;
 			infile= *(++argv);
 			}
+		else if (strcmp(*argv,"-nocrl") == 0)
+			{
+			nocrl=1;
+			}
 		else if (strcmp(*argv,"-out") == 0)
 			{
 			if (--argc < 1) goto bad;
@@ -134,7 +150,7 @@ char **argv;
 			}
 		else
 			{
-			fprintf(stderr,"unknown option %s\n",*argv);
+			BIO_printf(bio_err,"unknown option %s\n",*argv);
 			badops=1;
 			break;
 			}
@@ -145,13 +161,14 @@ char **argv;
 	if (badops)
 		{
 bad:
-		fprintf(stderr,"%s [options] <infile >outfile\n",prog);
-		fprintf(stderr,"where options are\n");
-		fprintf(stderr," -inform arg    input format - one of DER TXT PEM\n");
-		fprintf(stderr," -outform arg   output format - one of DER TXT PEM\n");
-		fprintf(stderr," -in arg        inout file\n");
-		fprintf(stderr," -out arg       output file\n");
-		fprintf(stderr," -certfile arg  certificates file of chain to a trusted CA\n");
+		BIO_printf(bio_err,"%s [options] <infile >outfile\n",prog);
+		BIO_printf(bio_err,"where options are\n");
+		BIO_printf(bio_err," -inform arg    input format - one of DER TXT PEM\n");
+		BIO_printf(bio_err," -outform arg   output format - one of DER TXT PEM\n");
+		BIO_printf(bio_err," -in arg        inout file\n");
+		BIO_printf(bio_err," -out arg       output file\n");
+		BIO_printf(bio_err," -certfile arg  certificates file of chain to a trusted CA\n");
+		BIO_printf(bio_err," -nocrl         no crl to load, just certs from '-certfile'\n");
 		EXIT(1);
 		}
 
@@ -165,30 +182,33 @@ bad:
 		goto end;
 		}
 
-	if (infile == NULL)
-		BIO_set_fp(in,stdin,BIO_NOCLOSE);
-	else
+	if (!nocrl)
 		{
-		if (BIO_read_filename(in,infile) <= 0)
+		if (infile == NULL)
+			BIO_set_fp(in,stdin,BIO_NOCLOSE);
+		else
 			{
-			perror(infile);
+			if (BIO_read_filename(in,infile) <= 0)
+				{
+				perror(infile);
+				goto end;
+				}
+			}
+
+		if 	(informat == FORMAT_ASN1)
+			crl=d2i_X509_CRL_bio(in,NULL);
+		else if (informat == FORMAT_PEM)
+			crl=PEM_read_bio_X509_CRL(in,NULL,NULL);
+		else	{
+			BIO_printf(bio_err,"bad input format specified for input crl\n");
+			goto end;
+			}
+		if (crl == NULL)
+			{
+			BIO_printf(bio_err,"unable to load CRL\n");
+			ERR_print_errors(bio_err);
 			goto end;
 			}
-		}
-
-	if 	(informat == FORMAT_ASN1)
-		crl=d2i_X509_CRL_bio(in,NULL);
-	else if (informat == FORMAT_PEM)
-		crl=PEM_read_bio_X509_CRL(in,NULL,NULL);
-	else	{
-		fprintf(stderr,"bad input format specified for input crl\n");
-		goto end;
-		}
-	if (crl == NULL)
-		{
-		fprintf(stderr,"unable to load CRL\n");
-		ERR_print_errors(bio_err);
-		goto end;
 		}
 	
 	if ((p7=PKCS7_new()) == NULL) goto end;
@@ -200,8 +220,11 @@ bad:
 	if (!ASN1_INTEGER_set(p7s->version,1)) goto end;
 	if ((crl_stack=sk_new(NULL)) == NULL) goto end;
 	p7s->crl=crl_stack;
-	sk_push(crl_stack,(char *)crl);
-	crl=NULL; /* now part of p7 for Freeing */
+	if (crl != NULL)
+		{
+		sk_push(crl_stack,(char *)crl);
+		crl=NULL; /* now part of p7 for Freeing */
+		}
 
 	if ((cert_stack=sk_new(NULL)) == NULL) goto end;
 	p7s->cert=cert_stack;
@@ -210,7 +233,7 @@ bad:
 		{
 		if (add_certs_from_file(cert_stack,certfile) < 0)
 			{
-			fprintf(stderr,"error loading certificates\n");
+			BIO_printf(bio_err,"error loading certificates\n");
 			ERR_print_errors(bio_err);
 			goto end;
 			}
@@ -232,12 +255,12 @@ bad:
 	else if (outformat == FORMAT_PEM)
 		i=PEM_write_bio_PKCS7(out,p7);
 	else	{
-		fprintf(stderr,"bad output format specified for outfile\n");
+		BIO_printf(bio_err,"bad output format specified for outfile\n");
 		goto end;
 		}
 	if (!i)
 		{
-		fprintf(stderr,"unable to write pkcs7 object\n");
+		BIO_printf(bio_err,"unable to write pkcs7 object\n");
 		ERR_print_errors(bio_err);
 		goto end;
 		}
@@ -274,14 +297,13 @@ char *certfile;
 
 	if ((stat(certfile,&st) != 0))
 		{
-		fprintf(stderr,"unable to file the file, %s\n",certfile);
+		BIO_printf(bio_err,"unable to file the file, %s\n",certfile);
 		goto end;
 		}
 
 	in=BIO_new(BIO_s_file());
 	if ((in == NULL) || (BIO_read_filename(in,certfile) <= 0))
 		{
-		SYSerr(ERR_F_FOPEN,errno);
 		goto end;
 		}
 
