@@ -1,17 +1,22 @@
 /* apps/sess_id.c */
-/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1997 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
+ *
+ * This package is an SSL implementation written
+ * by Eric Young (eay@cryptsoft.com).
+ * The implementation was written so as to conform with Netscapes SSL.
  * 
- * This file is part of an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
- * The implementation was written so as to conform with Netscapes SSL
- * specification.  This library and applications are
- * FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
- * as long as the following conditions are aheared to.
+ * This library is free for commercial and non-commercial use as long as
+ * the following conditions are aheared to.  The following conditions
+ * apply to all code found in this distribution, be it the RC4, RSA,
+ * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
+ * included with this distribution is covered by the same copyright terms
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
- * the code are not to be removed.  If this code is used in a product,
- * Eric Young should be given attribution as the author of the parts used.
+ * the code are not to be removed.
+ * If this package is used in a product, Eric Young should be given attribution
+ * as the author of the parts of the library used.
  * This can be in the form of a textual message at program startup or
  * in documentation (online or textual) provided with the package.
  * 
@@ -25,7 +30,13 @@
  *    documentation and/or other materials provided with the distribution.
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
- *    This product includes software developed by Eric Young (eay@mincom.oz.au)
+ *    "This product includes cryptographic software written by
+ *     Eric Young (eay@cryptsoft.com)"
+ *    The word 'cryptographic' can be left out if the rouines from the library
+ *    being used are not cryptographic related :-).
+ * 4. If you include any Windows specific code (or a derivative thereof) from 
+ *    the apps directory (application code) you must include an acknowledgement:
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -49,7 +60,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include "apps.h"
-#include "buffer.h"
+#include "bio.h"
 #include "err.h"
 #include "x509.h"
 #include "pem.h"
@@ -63,7 +74,7 @@
 #define FORMAT_TEXT	2
 #define FORMAT_PEM	3
 
-static char *ctl_usage[]={
+static char *sess_id_usage[]={
 "usage: crl args\n",
 "\n",
 " -inform arg     - input format - default PEM (one of DER, TXT or PEM)\n",
@@ -136,7 +147,7 @@ char **argv;
 			noout= ++num;
 		else
 			{
-			fprintf(stderr,"unknown option %s\n",*argv);
+			BIO_printf(bio_err,"unknown option %s\n",*argv);
 			badops=1;
 			break;
 			}
@@ -147,8 +158,8 @@ char **argv;
 	if (badops)
 		{
 bad:
-		for (pp=ctl_usage; (*pp != NULL); pp++)
-			fprintf(stderr,*pp);
+		for (pp=sess_id_usage; (*pp != NULL); pp++)
+			BIO_printf(bio_err,*pp);
 		goto end;
 		}
 
@@ -218,11 +229,11 @@ bad:
 		else if (outformat == FORMAT_PEM)
 			i=PEM_write_bio_SSL_SESSION(out,x);
 		else	{
-			fprintf(stderr,"bad output format specified for outfile\n");
+			BIO_printf(bio_err,"bad output format specified for outfile\n");
 			goto end;
 			}
 		if (!i) {
-			fprintf(stderr,"unable to write SSL_SESSION\n");
+			BIO_printf(bio_err,"unable to write SSL_SESSION\n");
 			goto end;
 			}
 		}
@@ -233,11 +244,11 @@ bad:
 		else if (outformat == FORMAT_PEM)
 			i=PEM_write_bio_X509(out,x->peer);
 		else	{
-			fprintf(stderr,"bad output format specified for outfile\n");
+			BIO_printf(bio_err,"bad output format specified for outfile\n");
 			goto end;
 			}
 		if (!i) {
-			fprintf(stderr,"unable to write X509\n");
+			BIO_printf(bio_err,"unable to write X509\n");
 			goto end;
 			}
 		}
@@ -277,12 +288,12 @@ int format;
 	else if (format == FORMAT_PEM)
 		x=PEM_read_bio_SSL_SESSION(in,NULL,NULL);
 	else	{
-		fprintf(stderr,"bad input format specified for input crl\n");
+		BIO_printf(bio_err,"bad input format specified for input crl\n");
 		goto end;
 		}
 	if (x == NULL)
 		{
-		fprintf(stderr,"unable to load SSL_SESSION\n");
+		BIO_printf(bio_err,"unable to load SSL_SESSION\n");
 		ERR_print_errors(bio_err);
 		goto end;
 		}
