@@ -74,6 +74,9 @@
 #include <signal.h>
 #include <string.h>
 #include "apps.h"
+#ifdef WIN16
+#define APPS_WIN16
+#endif
 #include "crypto.h"
 #include "rand.h"
 #include "err.h"
@@ -315,8 +318,12 @@ char **argv;
 	DSA *dsa_key[DSA_NUM];
 	long rsa_c[RSA_NUM][2];
 	long dsa_c[DSA_NUM][2];
+#ifndef NO_RSA
 	double rsa_results[RSA_NUM][2];
+#endif
+#ifndef NO_DSA
 	double dsa_results[DSA_NUM][2];
+#endif
 	static unsigned int rsa_bits[RSA_NUM]={512,1024,2048,4096};
 	static unsigned int dsa_bits[DSA_NUM]={512,1024,2048};
 	static unsigned char *rsa_data[RSA_NUM]=
@@ -324,7 +331,6 @@ char **argv;
 	static int rsa_data_length[RSA_NUM]={
 		sizeof(test512),sizeof(test1024),
 		sizeof(test2048),sizeof(test4096)};
-	unsigned char *p;
 	int doit[ALGOR_NUM];
 	int rsa_doit[RSA_NUM];
 	int dsa_doit[DSA_NUM];
@@ -478,7 +484,12 @@ char **argv;
 #ifndef NO_RC4
 			BIO_printf(bio_err,"rc4");
 #endif
+#ifndef NO_RSA
 			BIO_printf(bio_err,"\nrsa512   rsa1024  rsa2048  rsa4096\n");
+#endif
+#ifndef NO_DSA
+			BIO_printf(bio_err,"\ndsa512   dsa1024  dsa2048\n");
+#endif
 			BIO_printf(bio_err,"idea     rc2      des      rsa    blowfish\n");
 			goto end;
 			}
@@ -507,6 +518,8 @@ char **argv;
 #ifndef NO_RSA
 	for (i=0; i<RSA_NUM; i++)
 		{
+		unsigned char *p;
+
 		p=rsa_data[i];
 		rsa_key[i]=d2i_RSAPrivateKey(NULL,&p,rsa_data_length[i]);
 		if (rsa_key[i] == NULL)
