@@ -1,17 +1,22 @@
 /* crypto/asn1/a_type.c */
-/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1997 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
+ *
+ * This package is an SSL implementation written
+ * by Eric Young (eay@cryptsoft.com).
+ * The implementation was written so as to conform with Netscapes SSL.
  * 
- * This file is part of an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
- * The implementation was written so as to conform with Netscapes SSL
- * specification.  This library and applications are
- * FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
- * as long as the following conditions are aheared to.
+ * This library is free for commercial and non-commercial use as long as
+ * the following conditions are aheared to.  The following conditions
+ * apply to all code found in this distribution, be it the RC4, RSA,
+ * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
+ * included with this distribution is covered by the same copyright terms
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
- * the code are not to be removed.  If this code is used in a product,
- * Eric Young should be given attribution as the author of the parts used.
+ * the code are not to be removed.
+ * If this package is used in a product, Eric Young should be given attribution
+ * as the author of the parts of the library used.
  * This can be in the form of a textual message at program startup or
  * in documentation (online or textual) provided with the package.
  * 
@@ -25,7 +30,13 @@
  *    documentation and/or other materials provided with the distribution.
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
- *    This product includes software developed by Eric Young (eay@mincom.oz.au)
+ *    "This product includes cryptographic software written by
+ *     Eric Young (eay@cryptsoft.com)"
+ *    The word 'cryptographic' can be left out if the rouines from the library
+ *    being used are not cryptographic related :-).
+ * 4. If you include any Windows specific code (or a derivative thereof) from 
+ *    the apps directory (application code) you must include an acknowledgement:
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -52,8 +63,15 @@
 /* ASN1err(ASN1_F_ASN1_TYPE_NEW,ASN1_R_ERROR_STACK);
  * ASN1err(ASN1_F_D2I_ASN1_BYTES,ASN1_R_ERROR_STACK);
  * ASN1err(ASN1_F_D2I_ASN1_BYTES,ASN1_R_WRONG_TAG);
+ * ASN1err(ASN1_F_ASN1_COLLATE_PRIMATIVE,ASN1_R_WRONG_TAG);
  */
 
+#ifndef NOPROTO
+static void ASN1_TYPE_component_free(ASN1_TYPE *a);
+#else
+static void ASN1_TYPE_component_free();
+#endif
+
 int i2d_ASN1_TYPE(a,pp)
 ASN1_TYPE *a;
 unsigned char **pp;
@@ -83,20 +101,36 @@ unsigned char **pp;
 		r=i2d_ASN1_OBJECT(a->value.object,pp);
 		break;
 	case V_ASN1_PRINTABLESTRING:
-		r=i2d_ASN1_PRINTABLESTRING(a->value.printablestring,pp);
+		r=M_i2d_ASN1_PRINTABLESTRING(a->value.printablestring,pp);
 		break;
 	case V_ASN1_T61STRING:
-		r=i2d_ASN1_T61STRING(a->value.t61string,pp);
+		r=M_i2d_ASN1_T61STRING(a->value.t61string,pp);
 		break;
 	case V_ASN1_IA5STRING:
-		r=i2d_ASN1_IA5STRING(a->value.ia5string,pp);
+		r=M_i2d_ASN1_IA5STRING(a->value.ia5string,pp);
+		break;
+	case V_ASN1_GENERALSTRING:
+		r=M_i2d_ASN1_GENERALSTRING(a->value.generalstring,pp);
+		break;
+	case V_ASN1_UNIVERSALSTRING:
+		r=M_i2d_ASN1_UNIVERSALSTRING(a->value.universalstring,pp);
 		break;
 	case V_ASN1_UTCTIME:
 		r=i2d_ASN1_UTCTIME(a->value.utctime,pp);
 		break;
 	case V_ASN1_SET:
 	case V_ASN1_SEQUENCE:
-		r=i2d_ASN1_bytes(a->value.set,pp,a->type,V_ASN1_UNIVERSAL);
+		if (a->value.set == NULL)
+			r=0;
+		else
+			{
+			r=a->value.set->length;
+			if (pp != NULL)
+				{
+				memcpy(*pp,a->value.set->data,r);
+				*pp+=r;
+				}
+			}
 		break;
 		}
 	return(r);
@@ -124,11 +158,10 @@ long length;
 	max=(p+length);
 
 	inf=ASN1_get_object(&q,&len,&tag,&xclass,length);
-	if (inf == 0xff) goto err;
+	if (inf & 0x80) goto err;
 	
-	if (ret->value.ptr != NULL)
-		ASN1_TYPE_free(ret);
-	ret->value.ptr=NULL;
+	ASN1_TYPE_component_free(ret);
+
 	switch (tag)
 		{
 	case V_ASN1_NULL:
@@ -162,12 +195,22 @@ long length;
 		break;
 	case V_ASN1_T61STRING:
 		if ((ret->value.t61string=
-			d2i_ASN1_T61STRING(NULL,&p,max-p)) == NULL)
+			M_d2i_ASN1_T61STRING(NULL,&p,max-p)) == NULL)
 			goto err;
 		break;
 	case V_ASN1_IA5STRING:
 		if ((ret->value.ia5string=
-			d2i_ASN1_IA5STRING(NULL,&p,max-p)) == NULL)
+			M_d2i_ASN1_IA5STRING(NULL,&p,max-p)) == NULL)
+			goto err;
+		break;
+	case V_ASN1_GENERALSTRING:
+		if ((ret->value.generalstring=
+			M_d2i_ASN1_GENERALSTRING(NULL,&p,max-p)) == NULL)
+			goto err;
+		break;
+	case V_ASN1_UNIVERSALSTRING:
+		if ((ret->value.universalstring=
+			M_d2i_ASN1_UNIVERSALSTRING(NULL,&p,max-p)) == NULL)
 			goto err;
 		break;
 	case V_ASN1_UTCTIME:
@@ -177,10 +220,12 @@ long length;
 		break;
 	case V_ASN1_SET:
 	case V_ASN1_SEQUENCE:
-		if ((ret->value.set=
-			d2i_ASN1_bytes(NULL,&p,max-p,tag,V_ASN1_UNIVERSAL))
-			== NULL)
-			goto err;
+		/* Sets and sequences are left complete */
+		if ((ret->value.set=ASN1_STRING_new()) == NULL) goto err;
+		ret->value.set->type=tag;
+		len+=(q-p);
+		if (!ASN1_STRING_set(ret->value.set,p,(int)len)) goto err;
+		p+=len;
 		break;
 	default:
 		ASN1err(ASN1_F_D2I_ASN1_TYPE,ASN1_R_BAD_TYPE);
@@ -192,7 +237,7 @@ long length;
 	*pp=p;
 	return(ret);
 err:
-	if ((ret != NULL) && (a != NULL) && (*a != ret)) ASN1_TYPE_free(ret);
+	if ((ret != NULL) && ((a == NULL) || (*a != ret))) ASN1_TYPE_free(ret);
 	return(NULL);
 	}
 
@@ -208,6 +253,34 @@ ASN1_TYPE *ASN1_TYPE_new()
 	}
 
 void ASN1_TYPE_free(a)
+ASN1_TYPE *a;
+	{
+	if (a == NULL) return;
+	ASN1_TYPE_component_free(a);
+	Free((char *)(char *)a);
+	}
+
+int ASN1_TYPE_get(a)
+ASN1_TYPE *a;
+	{
+	if (a->value.ptr != NULL)
+		return(a->type);
+	else
+		return(0);
+	}
+
+void ASN1_TYPE_set(a,type,value)
+ASN1_TYPE *a;
+int type;
+char *value;
+	{
+	if (a->value.ptr != NULL)
+		ASN1_TYPE_component_free(a);
+	a->type=type;
+	a->value.ptr=value;
+	}
+
+static void ASN1_TYPE_component_free(a)
 ASN1_TYPE *a;
 	{
 	if (a == NULL) return;
@@ -216,126 +289,29 @@ ASN1_TYPE *a;
 		{
 		switch (a->type)
 			{
-		case V_ASN1_INTEGER:
-		case V_ASN1_NEG_INTEGER:
-			ASN1_INTEGER_free(a->value.integer);
-			break;
-		case V_ASN1_BIT_STRING:
-			ASN1_BIT_STRING_free(a->value.bit_string);
-			break;
-		case V_ASN1_OCTET_STRING:
-			ASN1_OCTET_STRING_free(a->value.octet_string);
-			break;
 		case V_ASN1_OBJECT:
 			ASN1_OBJECT_free(a->value.object);
 			break;
+		case V_ASN1_INTEGER:
+		case V_ASN1_NEG_INTEGER:
+		case V_ASN1_BIT_STRING:
+		case V_ASN1_OCTET_STRING:
 		case V_ASN1_PRINTABLESTRING:
-			ASN1_PRINTABLESTRING_free(a->value.printablestring);
-			break;
 		case V_ASN1_T61STRING:
-			ASN1_T61STRING_free(a->value.t61string);
-			break;
 		case V_ASN1_IA5STRING:
-			ASN1_IA5STRING_free(a->value.ia5string);
-			break;
+		case V_ASN1_UNIVERSALSTRING:
+		case V_ASN1_GENERALSTRING:
 		case V_ASN1_UTCTIME:
-			ASN1_UTCTIME_free(a->value.utctime);
-			break;
 		case V_ASN1_SET:
 		case V_ASN1_SEQUENCE:
-			ASN1_BIT_STRING_free(a->value.set);
+			ASN1_STRING_free((ASN1_STRING *)a->value.ptr);
+			break;
 		default:
 			/* MEMORY LEAK */
 			break;
 			}
+		a->type=0;
+		a->value.ptr=NULL;
 		}
-	Free((char *)(char *)a);
-	}
-
-int i2d_ASN1_bytes(a, pp, tag, xclass)
-ASN1_BIT_STRING *a;
-unsigned char **pp;
-int tag;
-int xclass;
-	{
-	int ret,r;
-	unsigned char *p;
-
-	if (a == NULL)  return(0);
-	ret=a->length;
-	r=ASN1_object_size(0,ret,tag);
-	if (pp == NULL) return(r);
-	p= *pp;
-
-	ASN1_put_object(&p,0,ret,tag,xclass);
-	memcpy(p,a->data,a->length);
-	p+=a->length;
-	*pp= p;
-	return(r);
-	}
-
-ASN1_BIT_STRING *d2i_ASN1_bytes(a, pp, length, Ptag, Pclass)
-ASN1_OCTET_STRING **a;
-unsigned char **pp;
-long length;
-int Ptag;
-int Pclass;
-	{
-	ASN1_BIT_STRING *ret=NULL;
-	unsigned char *p,*s;
-	long len;
-	int inf,tag,xclass;
-	int i=0;
-
-	if ((a == NULL) || ((*a) == NULL))
-		{
-		if ((ret=ASN1_BIT_STRING_new()) == NULL) return(NULL);
-		}
-	else
-		ret=(*a);
-
-	p= *pp;
-	inf=ASN1_get_object(&p,&len,&tag,&xclass,length);
-	if (inf == 0xff) goto err;
-
-	if (tag != Ptag)
-		{
-		i=ASN1_R_WRONG_TAG;
-		goto err;
-		}
-	if (len != 0)
-		{
-		if (ret->length < len)
-			{
-			if (ret->data != NULL) Free((char *)ret->data);
-			s=(unsigned char *)Malloc((int)len);
-			if (s == NULL)
-				{
-				i=ERR_R_MALLOC_FAILURE;
-				goto err;
-				}
-			}
-		else
-			s=ret->data;
-		memcpy(s,p,(int)len);
-		p+=len;
-		}
-	else
-		{
-		s=NULL;
-		if (ret->data != NULL) Free((char *)ret->data);
-		}
-
-	ret->length=(int)len;
-	ret->data=s;
-	ret->type=Ptag;
-	if (a != NULL) (*a)=ret;
-	*pp=p;
-	return(ret);
-err:
-	if ((ret != NULL) && (a != NULL) && (*a != ret))
-		ASN1_BIT_STRING_free(ret);
-	ASN1err(ASN1_F_D2I_ASN1_BYTES,i);
-	return(NULL);
 	}
 
