@@ -1,17 +1,22 @@
 /* crypto/asn1/t_pkey.c */
-/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1997 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
+ *
+ * This package is an SSL implementation written
+ * by Eric Young (eay@cryptsoft.com).
+ * The implementation was written so as to conform with Netscapes SSL.
  * 
- * This file is part of an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
- * The implementation was written so as to conform with Netscapes SSL
- * specification.  This library and applications are
- * FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
- * as long as the following conditions are aheared to.
+ * This library is free for commercial and non-commercial use as long as
+ * the following conditions are aheared to.  The following conditions
+ * apply to all code found in this distribution, be it the RC4, RSA,
+ * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
+ * included with this distribution is covered by the same copyright terms
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
- * the code are not to be removed.  If this code is used in a product,
- * Eric Young should be given attribution as the author of the parts used.
+ * the code are not to be removed.
+ * If this package is used in a product, Eric Young should be given attribution
+ * as the author of the parts of the library used.
  * This can be in the form of a textual message at program startup or
  * in documentation (online or textual) provided with the package.
  * 
@@ -25,7 +30,13 @@
  *    documentation and/or other materials provided with the distribution.
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
- *    This product includes software developed by Eric Young (eay@mincom.oz.au)
+ *    "This product includes cryptographic software written by
+ *     Eric Young (eay@cryptsoft.com)"
+ *    The word 'cryptographic' can be left out if the rouines from the library
+ *    being used are not cryptographic related :-).
+ * 4. If you include any Windows specific code (or a derivative thereof) from 
+ *    the apps directory (application code) you must include an acknowledgement:
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -49,11 +60,18 @@
 #include "cryptlib.h"
 #include "buffer.h"
 #include "bn.h"
-#include "dh.h"
+#ifndef NO_RSA
 #include "rsa.h"
+#endif
+#ifndef NO_DH
+#include "dh.h"
+#endif
+#ifndef NO_DSA
 #include "dsa.h"
+#endif
 
 /* DHerr(DH_F_DHPARAMS_PRINT,ERR_R_MALLOC_FAILURE);
+ * DSAerr(DSA_F_DSAPARAMS_PRINT,ERR_R_MALLOC_FAILURE);
  */
 
 #ifndef NOPROTO
@@ -63,6 +81,7 @@ static int print(BIO *fp,char *str,BIGNUM *num,
 static int print();
 #endif
 
+#ifndef NO_RSA
 #ifndef WIN16
 int RSA_print_fp(fp,x,off)
 FILE *fp;
@@ -74,7 +93,7 @@ int off;
 
         if ((b=BIO_new(BIO_s_file())) == NULL)
 		{
-		X509err(RSA_F_RSA_PRINT_FP,ERR_R_BUF_LIB);
+		RSAerr(RSA_F_RSA_PRINT_FP,ERR_R_BUF_LIB);
                 return(0);
 		}
         BIO_set_fp(b,fp,BIO_NOCLOSE);
@@ -94,7 +113,7 @@ int off;
 	int i,ret=0;
 
 	i=RSA_size(x);
-	m=(unsigned char *)malloc((unsigned int)i+10);
+	m=(unsigned char *)Malloc((unsigned int)i+10);
 	if (m == NULL)
 		{
 		RSAerr(RSA_F_RSA_PRINT,ERR_R_MALLOC_FAILURE);
@@ -109,12 +128,15 @@ int off;
 	if (x->d != NULL)
 		{
 		if (off && (BIO_write(bp,str,off) <= 0)) goto err;
-		sprintf(str,"Private-Key: (%d bit)\n",BN_num_bits(x->n));
-		if (BIO_puts(bp,str) <= 0) goto err;
+		if (BIO_printf(bp,"Private-Key: (%d bit)\n",BN_num_bits(x->n))
+			<= 0) goto err;
 		}
 
-	s=(x->d == NULL)?"Modulus:":"modulus:";
-	if (!print(bp,s,x->n,m,off)) goto err;
+	if (x->d == NULL)
+		sprintf(str,"Modulus (%d bit):",BN_num_bits(x->n));
+	else
+		strcpy(str,"modulus:");
+	if (!print(bp,str,x->n,m,off)) goto err;
 	s=(x->d == NULL)?"Exponent:":"publicExponent:";
 	if (!print(bp,s,x->e,m,off)) goto err;
 	if (!print(bp,"privateExponent:",x->d,m,off)) goto err;
@@ -125,9 +147,32 @@ int off;
 	if (!print(bp,"coefficient:",x->iqmp,m,off)) goto err;
 	ret=1;
 err:
-	if (m != NULL) free(m);
+	if (m != NULL) Free((char *)m);
 	return(ret);
 	}
+#endif /* NO_RSA */
+
+#ifndef NO_DSA
+#ifndef WIN16
+int DSA_print_fp(fp,x,off)
+FILE *fp;
+DSA *x;
+int off;
+	{
+	BIO *b;
+	int ret;
+
+	if ((b=BIO_new(BIO_s_file())) == NULL)
+		{
+		DSAerr(DSA_F_DSA_PRINT_FP,ERR_R_BUF_LIB);
+		return(0);
+		}
+	BIO_set_fp(b,fp,BIO_NOCLOSE);
+	ret=DSA_print(b,x,off);
+	BIO_free(b);
+	return(ret);
+	}
+#endif
 
 int DSA_print(bp,x,off)
 BIO *bp;
@@ -137,10 +182,21 @@ int off;
 	char str[128];
 	unsigned char *m=NULL;
 	int i,ret=0;
+	BIGNUM *bn=NULL;
 
+	if (x->p != NULL)
+		bn=x->p;
+	else if (x->priv_key != NULL)
+		bn=x->priv_key;
+	else if (x->pub_key != NULL)
+		bn=x->pub_key;
+		
 	/* larger than needed but what the hell :-) */
-	i=BN_num_bytes(x->g)*2;
-	m=(unsigned char *)malloc((unsigned int)i+10);
+	if (bn != NULL)
+		i=BN_num_bytes(bn)*2;
+	else
+		i=256;
+	m=(unsigned char *)Malloc((unsigned int)i+10);
 	if (m == NULL)
 		{
 		DSAerr(DSA_F_DSA_PRINT,ERR_R_MALLOC_FAILURE);
@@ -152,22 +208,26 @@ int off;
 		if (off > 128) off=128;
 		memset(str,' ',off);
 		}
-	if (x->x != NULL)
+	if (x->priv_key != NULL)
 		{
 		if (off && (BIO_write(bp,str,off) <= 0)) goto err;
-		sprintf(str,"Private-Key: (%d bit)\n",BN_num_bits(x->g));
+		if (BIO_printf(bp,"Private-Key: (%d bit)\n",BN_num_bits(x->p))
+			<= 0) goto err;
 		}
 
-	if (!print(bp,"X:",x->x,m,off)) goto err;
-	if (!print(bp,"Y:",x->y,m,off)) goto err;
-	if (!print(bp,"P:",x->p,m,off)) goto err;
-	if (!print(bp,"Q:",x->q,m,off)) goto err;
-	if (!print(bp,"G:",x->g,m,off)) goto err;
+	if ((x->priv_key != NULL) && !print(bp,"priv:",x->priv_key,m,off))
+		goto err;
+	if ((x->pub_key  != NULL) && !print(bp,"pub: ",x->pub_key,m,off))
+		goto err;
+	if ((x->p != NULL) && !print(bp,"P:   ",x->p,m,off)) goto err;
+	if ((x->q != NULL) && !print(bp,"Q:   ",x->q,m,off)) goto err;
+	if ((x->g != NULL) && !print(bp,"G:   ",x->g,m,off)) goto err;
 	ret=1;
 err:
-	if (m != NULL) free(m);
+	if (m != NULL) Free((char *)m);
 	return(ret);
 	}
+#endif /* !NO_DSA */
 
 static int print(bp,number,num,buf,off)
 BIO *bp;
@@ -177,8 +237,7 @@ unsigned char *buf;
 int off;
 	{
 	int n,i;
-	char *neg;
-	char str[128];
+	char str[128],*neg;
 
 	if (num == NULL) return(1);
 	neg=(num->neg)?"-":"";
@@ -191,15 +250,16 @@ int off;
 
 	if (BN_num_bytes(num) <= BN_BYTES)
 		{
-		sprintf(str,"%s %s%lu (%s0x%lx)\n",number,neg,
-			(unsigned long)num->d[0],neg,(unsigned long)num->d[0]);
-		if (BIO_puts(bp,str) <= 0) return(0);
+		if (BIO_printf(bp,"%s %s%lu (%s0x%lx)\n",number,neg,
+			(unsigned long)num->d[0],neg,(unsigned long)num->d[0])
+			<= 0) return(0);
 		}
 	else
 		{
 		buf[0]=0;
-		sprintf(str,"%s%s",number,(neg[0] == '-')?" (Negative)":"");
-		if (BIO_puts(bp,str) <= 0) return(0);
+		if (BIO_printf(bp,"%s%s",number,
+			(neg[0] == '-')?" (Negative)":"") <= 0)
+			return(0);
 		n=BN_bn2bin(num,&buf[1]);
 	
 		if (buf[1] & 0x80)
@@ -214,14 +274,15 @@ int off;
 				memset(&(str[1]),' ',off+4);
 				if (BIO_write(bp,str,off+1+4) <= 0) return(0);
 				}
-			sprintf(str,"%02x%s",buf[i],((i+1) == n)?"":":");
-			if (BIO_puts(bp,str) <= 0) return(0);
+			if (BIO_printf(bp,"%02x%s",buf[i],((i+1) == n)?"":":")
+				<= 0) return(0);
 			}
-		if (BIO_puts(bp,"\n") <= 0) return(0);
+		if (BIO_write(bp,"\n",1) <= 0) return(0);
 		}
 	return(1);
 	}
 
+#ifndef NO_DH
 #ifndef WIN16
 int DHparams_print_fp(fp,x)
 FILE *fp;
@@ -247,30 +308,82 @@ BIO *bp;
 DH *x;
 	{
 	unsigned char *m=NULL;
-	char buf[128];
 	int reason=ERR_R_BUF_LIB,i,ret=0;
 
 	i=BN_num_bytes(x->p);
-	m=(unsigned char *)malloc((unsigned int)i+10);
+	m=(unsigned char *)Malloc((unsigned int)i+10);
 	if (m == NULL)
 		{
 		reason=ERR_R_MALLOC_FAILURE;
 		goto err;
 		}
 
-	sprintf(buf,"Diffie-Hellman-Parameters: (%d bit)\n",BN_num_bits(x->p));
-	if (BIO_puts(bp,buf) <= 0) goto err;
+	if (BIO_printf(bp,"Diffie-Hellman-Parameters: (%d bit)\n",
+		BN_num_bits(x->p)) <= 0)
+		goto err;
 	if (!print(bp,"prime:",x->p,m,4)) goto err;
 	if (!print(bp,"generator:",x->g,m,4)) goto err;
 	if (x->length != 0)
 		{
-		sprintf(buf,"    recomented-private-length: %d bits\n",
-			(int)x->length);
-		if (BIO_puts(bp,buf) <= 0) goto err;
+		if (BIO_printf(bp,"    recomented-private-length: %d bits\n",
+			(int)x->length) <= 0) goto err;
 		}
 	ret=1;
 err:
-	if (m != NULL) free(m);
+	if (m != NULL) Free((char *)m);
 	DHerr(DH_F_DHPARAMS_PRINT,reason);
 	return(ret);
 	}
+#endif
+
+#ifndef NO_DSA
+#ifndef WIN16
+int DSAparams_print_fp(fp,x)
+FILE *fp;
+DSA *x;
+        {
+        BIO *b;
+        int ret;
+
+        if ((b=BIO_new(BIO_s_file())) == NULL)
+		{
+		DSAerr(DSA_F_DSAPARAMS_PRINT_FP,ERR_R_BUF_LIB);
+                return(0);
+		}
+        BIO_set_fp(b,fp,BIO_NOCLOSE);
+        ret=DSAparams_print(b, x);
+        BIO_free(b);
+        return(ret);
+        }
+#endif
+
+int DSAparams_print(bp,x)
+BIO *bp;
+DSA *x;
+	{
+	unsigned char *m=NULL;
+	int reason=ERR_R_BUF_LIB,i,ret=0;
+
+	i=BN_num_bytes(x->p);
+	m=(unsigned char *)Malloc((unsigned int)i+10);
+	if (m == NULL)
+		{
+		reason=ERR_R_MALLOC_FAILURE;
+		goto err;
+		}
+
+	if (BIO_printf(bp,"DSA-Parameters: (%d bit)\n",
+		BN_num_bits(x->p)) <= 0)
+		goto err;
+	if (!print(bp,"p:",x->p,m,4)) goto err;
+	if (!print(bp,"q:",x->q,m,4)) goto err;
+	if (!print(bp,"g:",x->g,m,4)) goto err;
+	ret=1;
+err:
+	if (m != NULL) Free((char *)m);
+	DSAerr(DSA_F_DSAPARAMS_PRINT,reason);
+	return(ret);
+	}
+
+#endif /* !NO_DSA */
+
