@@ -1,17 +1,22 @@
 /* crypto/dsa/dsa_vrf.c */
-/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1997 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
+ *
+ * This package is an SSL implementation written
+ * by Eric Young (eay@cryptsoft.com).
+ * The implementation was written so as to conform with Netscapes SSL.
  * 
- * This file is part of an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
- * The implementation was written so as to conform with Netscapes SSL
- * specification.  This library and applications are
- * FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
- * as long as the following conditions are aheared to.
+ * This library is free for commercial and non-commercial use as long as
+ * the following conditions are aheared to.  The following conditions
+ * apply to all code found in this distribution, be it the RC4, RSA,
+ * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
+ * included with this distribution is covered by the same copyright terms
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
- * the code are not to be removed.  If this code is used in a product,
- * Eric Young should be given attribution as the author of the parts used.
+ * the code are not to be removed.
+ * If this package is used in a product, Eric Young should be given attribution
+ * as the author of the parts of the library used.
  * This can be in the form of a textual message at program startup or
  * in documentation (online or textual) provided with the package.
  * 
@@ -25,7 +30,13 @@
  *    documentation and/or other materials provided with the distribution.
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
- *    This product includes software developed by Eric Young (eay@mincom.oz.au)
+ *    "This product includes cryptographic software written by
+ *     Eric Young (eay@cryptsoft.com)"
+ *    The word 'cryptographic' can be left out if the rouines from the library
+ *    being used are not cryptographic related :-).
+ * 4. If you include any Windows specific code (or a derivative thereof) from 
+ *    the apps directory (application code) you must include an acknowledgement:
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -77,7 +88,7 @@ DSA *dsa;
 	BIGNUM *r=NULL;
 	BIGNUM *t1=NULL,*t2=NULL;
 	BIGNUM *u1=NULL,*u2=NULL;
-	ASN1_BIT_STRING *bs=NULL;
+	ASN1_INTEGER *bs=NULL;
 	int ret = -1;
 
 	ctx=BN_CTX_new();
@@ -85,7 +96,7 @@ DSA *dsa;
 
 	t1=BN_new();
 	t2=BN_new();
-	if (r == NULL || t1 == NULL || t2 == NULL || u1 == NULL) goto err;
+	if (t1 == NULL || t2 == NULL) goto err;
 
 	M_ASN1_D2I_Init();
 	M_ASN1_D2I_start_sequence();
@@ -95,28 +106,28 @@ DSA *dsa;
         if ((u1=BN_bin2bn(bs->data,bs->length,NULL)) == NULL) goto err_bn;
 	if (!asn1_Finish(&c)) goto err;
 
-	/* Calculate W = inv(S) mod Q */
-	u2 = BN_mod_inverse(u1,dsa->q,ctx);	/* save W in u2 */
-	if (u2 == NULL) goto err;
+	/* Calculate W = inv(S) mod Q
+	 * save W in u2 */
+	if ((u2=BN_mod_inverse(u1,dsa->q,ctx)) == NULL) goto err_bn;
 
 	/* save M in u1 */
-	if (BN_bin2bn(dgst,dgst_len,u1) == NULL) goto err;
+	if (BN_bin2bn(dgst,dgst_len,u1) == NULL) goto err_bn;
 
 	/* u1 = M * w mod q */
-	if (!BN_mod_mul(u1,u1,u2,dsa->q,ctx)) goto err;
+	if (!BN_mod_mul(u1,u1,u2,dsa->q,ctx)) goto err_bn;
 
 	/* u2 = r * w mod q */
-	if (!BN_mod_mul(u2,r,u2,dsa->q,ctx)) goto err;
+	if (!BN_mod_mul(u2,r,u2,dsa->q,ctx)) goto err_bn;
 
 	/* v = ( g^u1 * y^u2 mod p ) mod q */
 	/* let t1 = g ^ u1 mod p */
-	if (!BN_mod_exp(t1,dsa->g,u1,dsa->p,ctx)) goto err;
+	if (!BN_mod_exp(t1,dsa->g,u1,dsa->p,ctx)) goto err_bn;
 	/* let t2 = y ^ u2 mod p */
-	if (!BN_mod_exp(t2,dsa->y,u2,dsa->p,ctx)) goto err;
+	if (!BN_mod_exp(t2,dsa->pub_key,u2,dsa->p,ctx)) goto err_bn;
 	/* let u1 = t1 * t2 mod p */
-	if (!BN_mod_mul(u1,t1,t2,dsa->p,ctx)) goto err;
+	if (!BN_mod_mul(u1,t1,t2,dsa->p,ctx)) goto err_bn;
 	/* let u1 = u1 mod q */
-	if (!BN_mod(u1,u1,dsa->q,ctx)) goto err;
+	if (!BN_mod(u1,u1,dsa->q,ctx)) goto err_bn;
 	/* V is now in u1.  If the signature is correct, it will be
 	 * equal to R. */
 	ret=(BN_ucmp(u1, r) == 0);
