@@ -1,17 +1,22 @@
 /* crypto/evp/e_ecb_r2.c */
-/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1997 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
+ *
+ * This package is an SSL implementation written
+ * by Eric Young (eay@cryptsoft.com).
+ * The implementation was written so as to conform with Netscapes SSL.
  * 
- * This file is part of an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
- * The implementation was written so as to conform with Netscapes SSL
- * specification.  This library and applications are
- * FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
- * as long as the following conditions are aheared to.
+ * This library is free for commercial and non-commercial use as long as
+ * the following conditions are aheared to.  The following conditions
+ * apply to all code found in this distribution, be it the RC4, RSA,
+ * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
+ * included with this distribution is covered by the same copyright terms
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
- * the code are not to be removed.  If this code is used in a product,
- * Eric Young should be given attribution as the author of the parts used.
+ * the code are not to be removed.
+ * If this package is used in a product, Eric Young should be given attribution
+ * as the author of the parts of the library used.
  * This can be in the form of a textual message at program startup or
  * in documentation (online or textual) provided with the package.
  * 
@@ -25,7 +30,13 @@
  *    documentation and/or other materials provided with the distribution.
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
- *    This product includes software developed by Eric Young (eay@mincom.oz.au)
+ *    "This product includes cryptographic software written by
+ *     Eric Young (eay@cryptsoft.com)"
+ *    The word 'cryptographic' can be left out if the rouines from the library
+ *    being used are not cryptographic related :-).
+ * 4. If you include any Windows specific code (or a derivative thereof) from 
+ *    the apps directory (application code) you must include an acknowledgement:
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -49,12 +60,12 @@
 
 #include <stdio.h>
 #include "cryptlib.h"
-#include "envelope.h"
+#include "evp.h"
 #include "objects.h"
 
 #ifndef NOPROTO
 static void rc2_ecb_init_key(EVP_CIPHER_CTX *ctx, unsigned char *key,
-	unsigned char *iv);
+	unsigned char *iv,int enc);
 static void rc2_ecb_cipher(EVP_CIPHER_CTX *ctx, unsigned char *out,
 	unsigned char *in, unsigned int inl);
 #else
@@ -67,7 +78,6 @@ static EVP_CIPHER r2_ecb_cipher=
 	NID_rc2_ecb,
 	8,EVP_RC2_KEY_SIZE,0,
 	rc2_ecb_init_key,
-	rc2_ecb_init_key,
 	rc2_ecb_cipher,
 	};
 
@@ -76,13 +86,15 @@ EVP_CIPHER *EVP_rc2_ecb()
 	return(&r2_ecb_cipher);
 	}
 	
-static void rc2_ecb_init_key(ctx,key,iv)
+static void rc2_ecb_init_key(ctx,key,iv,enc)
 EVP_CIPHER_CTX *ctx;
 unsigned char *key;
 unsigned char *iv;
+int enc;
 	{
 	if (key != NULL)
-		RC2_set_key(&(ctx->c.rc2_ecb.ks),EVP_RC2_KEY_SIZE,key);
+		RC2_set_key(&(ctx->c.rc2_ecb.ks),EVP_RC2_KEY_SIZE,key,
+			EVP_RC2_KEY_SIZE*8);
 	}
 
 static void rc2_ecb_cipher(ctx,out,in,inl)
