@@ -290,11 +290,18 @@ int inl;
 		if ((v == B64_EOF) || (n >= 64))
 			{
 			tmp2=v;
-			v=EVP_DecodeBlock(out,d,n);
-			if (v < 0) { rv=0; goto end; }
-
-			n=0;
-			ret+=(v-eof);
+			if (n > 0)
+				{
+				v=EVP_DecodeBlock(out,d,n);
+				if (v < 0) { rv=0; goto end; }
+				n=0;
+				ret+=(v-eof);
+				}
+			else
+				{
+				eof=1;
+				v=0;
+				}
 
 			/* This is the case where we have had a short
 			 * but valid input line */
