@@ -1,9 +1,9 @@
 /* crypto/lhash/lhash.c */
-/* Copyright (C) 1995-1997 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1997 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
  * This package is an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
+ * by Eric Young (eay@cryptsoft.com).
  * The implementation was written so as to conform with Netscapes SSL.
  * 
  * This library is free for commercial and non-commercial use as long as
@@ -11,7 +11,7 @@
  * apply to all code found in this distribution, be it the RC4, RSA,
  * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
  * included with this distribution is covered by the same copyright terms
- * except that the holder is Tim Hudson (tjh@mincom.oz.au).
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
  * the code are not to be removed.
@@ -31,12 +31,12 @@
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
  *    "This product includes cryptographic software written by
- *     Eric Young (eay@mincom.oz.au)"
+ *     Eric Young (eay@cryptsoft.com)"
  *    The word 'cryptographic' can be left out if the rouines from the library
  *    being used are not cryptographic related :-).
  * 4. If you include any Windows specific code (or a derivative thereof) from 
  *    the apps directory (application code) you must include an acknowledgement:
- *    "This product includes software written by Tim Hudson (tjh@mincom.oz.au)"
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -56,7 +56,7 @@
  * [including the GNU Public Licence.]
  */
 
-char *lh_version="lhash part of SSLeay 0.6.6b 29-Jun-1998";
+char *lh_version="lhash part of SSLeay 0.8.1a 24-Jun-1998";
 
 /* Code for dynamic hash table routines
  * Author - Eric Young v 2.0
@@ -70,7 +70,7 @@ char *lh_version="lhash part of SSLeay 0.6.6b 29-Jun-1998";
  * 1.9 eay - Fixed a memory leak in lh_free, the LHASH_NODEs
  *	     were not being free()ed. 21/11/95
  *
- * 1.8 eay - Put the stats routines into a seperate file, lh_stats.c
+ * 1.8 eay - Put the stats routines into a separate file, lh_stats.c
  *	     19/09/95
  *
  * 1.7 eay - Removed the fputs() for realloc failures - the code
@@ -152,8 +152,8 @@ int (*c)();
 	ret->num_replace=0;
 	ret->num_delete=0;
 	ret->num_no_delete=0;
-	ret->num_retreve=0;
-	ret->num_retreve_miss=0;
+	ret->num_retrieve=0;
+	ret->num_retrieve_miss=0;
 	ret->num_hash_comps=0;
 
 	return(ret);
@@ -263,13 +263,13 @@ char *data;
 
 	if (*rn == NULL)
 		{
-		lh->num_retreve_miss++;
+		lh->num_retrieve_miss++;
 		return(NULL);
 		}
 	else
 		{
 		ret= (*rn)->data;
-		lh->num_retreve++;
+		lh->num_retrieve++;
 		}
 	return(ret);
 	}
