@@ -155,7 +155,12 @@ SSL *s;
 		case SSL_ST_ACCEPT:
 		case SSL_ST_BEFORE|SSL_ST_ACCEPT:
 		case SSL_ST_OK|SSL_ST_ACCEPT:
+
+			if (cb != NULL) cb(s,SSL_CB_HANDSHAKE_START,1);
+
 			s->version=2;
+			s->type=SSL_ST_ACCEPT;
+
 			buf=s->init_buf;
 			if ((buf == NULL) && ((buf=BUF_MEM_new()) == NULL))
 				{ ret= -1; goto end; }
@@ -301,6 +306,8 @@ SSL *s;
 			/* s->server=1; */
 			ret=1;
 
+			if (cb != NULL) cb(s,SSL_CB_HANDSHAKE_DONE,1);
+
 			goto end;
 			BREAK;
 
@@ -329,7 +336,7 @@ end:
 static int get_client_master_key(s)
 SSL *s;
 	{
-	int export,i,n,keya,error=0,ek;
+	int export,i,n,keya,ek;
 	unsigned char *p;
 	SSL_CIPHER *cp;
 	EVP_CIPHER *c;
@@ -395,19 +402,37 @@ SSL *s;
 
 	export=(s->session->cipher->algorithms & SSL_EXP)?1:0;
 	
-	if (!ssl_session_get_ciphers(s->session,&c,&md))
+	if (!ssl_cipher_get_evp(s->session->cipher,&c,&md))
 		{
 		ssl2_return_error(s,SSL2_PE_NO_CIPHER);
 		SSLerr(SSL_F_GET_CLIENT_MASTER_KEY,SSL_R_PROBLEMS_MAPPING_CIPHER_FUNCTIONS);
 		return(0);
 		}
 
-	if (export)
-		ek=(s->session->cipher->algorithm2 & SSL2_CF_8_BYTE_ENC)?8:5;
+	if (s->session->cipher->algorithm2 & SSL2_CF_8_BYTE_ENC)
+		{
+		export=1;
+		ek=8;
+		}
 	else
 		ek=5;
 
 	/* bad decrypt */
+#if 1
+	/* If a bad decrypt, continue with protocol but with a
+	 * dud master secret */
+	if ((i < 0) ||
+		((!export && (i != EVP_CIPHER_key_length(c)))
+		|| ( export && ((i != ek) || (s->s2->tmp.clear+i !=
+			EVP_CIPHER_key_length(c))))))
+		{
+		if (export)
+			i=ek;
+		else
+			i=EVP_CIPHER_key_length(c);
+		RAND_bytes(p,i);
+		}
+#else
 	if (i < 0)
 		{
 		error=1;
@@ -426,6 +451,7 @@ SSL *s;
 		ssl2_return_error(s,SSL2_PE_UNDEFINED_ERROR);
 		return(-1);
 		}
+#endif
 
 	if (export) i+=s->s2->tmp.clear;
 	s->session->master_key_length=i;
@@ -446,7 +472,7 @@ SSL *s;
 	 * type the first time round. */
 	if (s->state == SSL2_ST_GET_CLIENT_HELLO_A)
 		{
-		s->s2->first_packet=1;
+		s->first_packet=1;
 		s->state=SSL2_ST_GET_CLIENT_HELLO_B;
 		}
 
