@@ -60,7 +60,7 @@
 #include "objects.h"
 #include "ssl_locl.h"
 
-char *ssl3_version_str="SSLv3 part of SSLeay 0.8.0 25-Jun-1997";
+char *ssl3_version_str="SSLv3 part of SSLeay 0.8.1a 24-Jun-1998";
 
 #define SSL3_NUM_CIPHERS	(sizeof(ssl3_ciphers)/sizeof(SSL_CIPHER))
 
@@ -595,7 +595,7 @@ unsigned char *p;
 			FP_ICC ssl_cipher_ptr_id_cmp);
 		}
 
-	id=0x03000000|(p[0]<<8L)|p[1];
+	id=0x03000000L|((unsigned long)p[0]<<8L)|(unsigned long)p[1];
 	c.id=id;
 	cpp=(SSL_CIPHER **)OBJ_bsearch((char *)&cp,
 		(char *)sorted,
@@ -774,8 +774,10 @@ unsigned char *p;
 int ssl3_shutdown(s)
 SSL *s;
 	{
-	/* Don't do anything much if we have not done the handshake */
-	if (s->state == SSL_ST_BEFORE)
+
+	/* Don't do anything much if we have not done the handshake or
+	 * we don't want to send messages :-) */
+	if ((s->quiet_shutdown) || (s->state == SSL_ST_BEFORE))
 		{
 		s->shutdown=(SSL_SENT_SHUTDOWN|SSL_RECEIVED_SHUTDOWN);
 		return(1);
