@@ -170,13 +170,17 @@ SSL *s;
 			{
 		case SSL_ST_RENEGOTIATE:
 			s->new_session=1;
-			s->state=SSL_ST_ACCEPT;
+			/* s->state=SSL_ST_ACCEPT; */
 
 		case SSL_ST_BEFORE:
 		case SSL_ST_ACCEPT:
 		case SSL_ST_BEFORE|SSL_ST_ACCEPT:
 		case SSL_ST_OK|SSL_ST_ACCEPT:
+
+			if (cb != NULL) cb(s,SSL_CB_HANDSHAKE_START,1);
+
 			s->version=3;
+			s->type=SSL_ST_ACCEPT;
 
 			if (s->init_buf == NULL)
 				{
@@ -245,15 +249,31 @@ SSL *s;
 			s->shutdown=0;
 			ret=ssl3_send_hello_request(s);
 			if (ret <= 0) goto end;
-			s->s3->tmp.next_state=SSL3_ST_SR_CLNT_HELLO_A;
+			s->s3->tmp.next_state=SSL3_ST_SW_HELLO_REQ_C;
 			s->state=SSL3_ST_SW_FLUSH;
 			s->init_num=0;
 
 			ssl3_init_finished_mac(s);
 			break;
 
+		case SSL3_ST_SW_HELLO_REQ_C:
+			/* remove buffering on output */
+			under=BIO_pop(s->wbio);
+			if (under != NULL)
+				s->wbio=under;
+			else
+				abort(); /* ok */
+			BIO_free(s->bbio);
+			s->bbio=NULL;
+
+			s->state=SSL_ST_OK;
+			ret=1;
+			goto end;
+			break;
+
 		case SSL3_ST_SR_CLNT_HELLO_A:
 		case SSL3_ST_SR_CLNT_HELLO_B:
+		case SSL3_ST_SR_CLNT_HELLO_C:
 
 			s->shutdown=0;
 			ret=ssl3_get_client_hello(s);
@@ -490,6 +510,8 @@ SSL *s;
 			s->handshake_func=ssl3_accept;
 			ret=1;
 
+			if (cb != NULL) cb(s,SSL_CB_HANDSHAKE_DONE,1);
+
 			goto end;
 			break;
 
@@ -556,9 +578,14 @@ SSL *s;
 	SSL_CIPHER *c;
 	STACK *ciphers=NULL;
 
+	if (s->state == SSL3_ST_SR_CLNT_HELLO_A)
+		{
+		s->first_packet=1;
+		s->state=SSL3_ST_SR_CLNT_HELLO_B;
+		}
 	n=ssl3_get_message(s,
-		SSL3_ST_SR_CLNT_HELLO_A,
 		SSL3_ST_SR_CLNT_HELLO_B,
+		SSL3_ST_SR_CLNT_HELLO_C,
 		SSL3_MT_CLIENT_HELLO,
 		SSL3_RT_MAX_PLAIN_LENGTH,
 		&ok);
@@ -566,7 +593,7 @@ SSL *s;
 	if (!ok) return((int)n);
 	d=p=(unsigned char *)s->init_buf->data;
 
-	if ((p[0] != SSL3_VERSION_MAJOR) && (p[1] != SSL3_VERSION_MINOR))
+	if (p[0] != SSL3_VERSION_MAJOR)
 		{
 		SSLerr(SSL_F_SSL3_GET_CLIENT_HELLO,SSL_R_WRONG_SSL_VERSION);
 		goto err;
@@ -850,19 +877,25 @@ SSL *s;
 static int ssl3_send_server_key_exchange(s)
 SSL *s;
 	{
-	unsigned char *p,*d,*q;
-	int al,num,i,j;
+#ifndef NO_RSA
+	unsigned char *q;
+	int j,num;
+	RSA *rsa;
+	unsigned char md_buf[MD5_DIGEST_LENGTH+SHA_DIGEST_LENGTH];
+#endif
+#ifndef NO_DH
+	DH *dh,*dhp;
+#endif
+	EVP_PKEY *pkey;
+	unsigned char *p,*d;
+	int al,i;
 	unsigned long type;
 	int n;
 	CERT *cert;
-	RSA *rsa;
-	DH *dh,*dhp;
 	BIGNUM *r[4];
 	int nr[4],kn;
 	BUF_MEM *buf;
-	EVP_PKEY *pkey;
 	EVP_MD_CTX md_ctx;
-	unsigned char md_buf[MD5_DIGEST_LENGTH+SHA_DIGEST_LENGTH];
 
 	if (s->state == SSL3_ST_SW_KEY_EXCH_A)
 		{
@@ -1149,7 +1182,7 @@ SSL *s;
 	unsigned char *p;
 	RSA *rsa=NULL;
 	BIGNUM *pub=NULL;
-	EVP_PKEY *pkey;
+	EVP_PKEY *pkey=NULL;
 	DH *dh_srvr;
 
 	n=ssl3_get_message(s,
@@ -1201,14 +1234,33 @@ SSL *s;
 			}
 
 		i=RSA_private_decrypt((int)n,p,p,rsa,RSA_PKCS1_PADDING);
-
-		if (i <= 0)
+#if 1
+		/* If a bad decrypt, use a dud master key */
+		if ((i != SSL_MAX_MASTER_KEY_LENGTH) ||
+			(p[0] != 3) ||
+			 (p[1] != 0))
 			{
-			al=SSL3_AD_HANDSHAKE_FAILURE;
+			p[0]=3;
+			p[1]=0;
+			RAND_bytes(&(p[2]),SSL_MAX_MASTER_KEY_LENGTH-2);
+			i=SSL_MAX_MASTER_KEY_LENGTH;
+			}
+#else
+		if (i != SSL_MAX_MASTER_KEY_LENGTH)
+			{
+			al=SSL_AD_DECODE_ERROR;
 			SSLerr(SSL_F_SSL3_GET_CLIENT_KEY_EXCHANGE,SSL_R_BAD_RSA_DECRYPT);
 			goto f_err;
 			}
 
+		if ((p[0] != (s->version>>8)) || (p[1] != (s->version & 0xff)))
+			{
+			al=SSL_AD_DECODE_ERROR;
+			SSLerr(SSL_F_SSL3_GET_CLIENT_KEY_EXCHANGE,SSL_R_BAD_PROTOCOL_VERSION_NUMBER);
+			goto f_err;
+			}
+#endif
+
 		s->session->master_key_length=
 			ssl3_generate_master_secret(s,
 				s->session->master_key,
@@ -1220,6 +1272,22 @@ SSL *s;
 #ifndef NO_DH
 		if (l & (SSL_kEDH|SSL_kDHr|SSL_kDHd))
 		{
+
+		n2s(p,i);
+		if (n != i+2)
+			{
+			if (!(s->ctx->options & SSL_OP_SSLEAY_080_CLIENT_DH_BUG))
+				{
+				SSLerr(SSL_F_SSL3_GET_CLIENT_KEY_EXCHANGE,SSL_R_DH_PUBLIC_VALUE_LENGTH_IS_WRONG);
+				goto err;
+				}
+			else
+				{
+				p-=2;
+				i=(int)n;
+				}
+			}
+
 		if (n == 0L) /* the parameters are in the cert */
 			{
 			al=SSL3_AD_HANDSHAKE_FAILURE;
@@ -1237,13 +1305,15 @@ SSL *s;
 			else
 				dh_srvr=s->s3->tmp.dh;
 			}
-		pub=BN_bin2bn(p,(int)n,NULL);
+
+		pub=BN_bin2bn(p,i,NULL);
 		if (pub == NULL)
 			{
 			SSLerr(SSL_F_SSL3_GET_CLIENT_KEY_EXCHANGE,SSL_R_BN_LIB);
 			goto err;
 			}
 		i=DH_compute_key(p,pub,dh_srvr);
+
 		if (i <= 0)
 			{
 			SSLerr(SSL_F_SSL3_GET_CLIENT_KEY_EXCHANGE,ERR_R_DH_LIB);
