@@ -138,6 +138,10 @@ extern "C" {
 
 #define SSL_SENT_SHUTDOWN	1
 #define SSL_RECEIVED_SHUTDOWN	2
+#define SSL_CTX_set_quiet_shutdown(ctx,y) ((ctx)->quiet_shutdown=(y));
+#define SSL_CTX_get_quiet_shutdown(ctx) ((ctx)->quiet_shutdown);
+#define SSL_set_quiet_shutdown(s,y)	((s)->quiet_shutdown=(y));
+#define SSL_get_quiet_shutdown(s)	((s)->quiet_shutdown);
 #define SSL_set_shutdown(s,mode)	((s)->shutdown=(mode))
 #define SSL_get_shutdown(s)		((s)->shutdown)
 #define SSL_version(s)			((s)->version)
@@ -254,17 +258,26 @@ typedef struct ssl_session_st
 #define SSL_OP_SSLREF2_REUSE_CERT_TYPE_BUG		0x00000010L
 #define SSL_OP_MICROSOFT_BIG_SSLV3_BUFFER		0x00000020L
 #define SSL_OP_MSIE_SSLV2_RSA_PADDING			0x00000040L
+#define SSL_OP_SSLEAY_080_CLIENT_DH_BUG			0x00000080L
 
 /* If set, only use tmp_dh parameters once */
 #define SSL_OP_SINGLE_DH_USE				0x00100000L
 /* Set to also use the tmp_rsa key when doing RSA operations. */
 #define SSL_OP_EPHEMERAL_RSA				0x00200000L
 
+#define SSL_OP_NON_EXPORT_FIRST 			0x40000000L
 #define SSL_OP_NETSCAPE_DEMO_CIPHER_CHANGE_BUG		0x80000000L
 #define SSL_OP_ALL					0x000FFFFFL
 
 #define SSL_CTX_set_options(ctx,op)	((ctx)->options|=(op))
 
+/* Normally you will only use these if your application wants to use
+ * the certificate store in other places, perhaps PKCS7 */
+#define SSL_CTX_get_cert_store(ctx)     ((ctx)->cert_store)
+#define SSL_CTX_set_cert_store(ctx,cs) \
+                (X509_STORE_free((ctx)->cert_store),(ctx)->cert_store=(cs))
+
+
 typedef struct ssl_ctx_st
 	{
 	SSL_METHOD *method;
@@ -345,6 +358,8 @@ typedef struct ssl_ctx_st
 	/* what we put in client requests */
 	STACK *client_CA;
 
+	int quiet_shutdown;
+
 	char *app_data;
 	} SSL_CTX;
 
@@ -405,6 +420,7 @@ typedef struct ssl_st
 	 * 3 for SSLv3
 	 * -3 for SSLv3 but accept SSLv2 */
 	int version;
+	int type; /* SSL_ST_CONNECT or SSL_ST_ACCEPT */
 
 	SSL_METHOD *method; /* SSLv3 */
 
@@ -435,6 +451,7 @@ typedef struct ssl_st
 /*	int server;*/	/* are we the server side? */
 
 	int new_session;/* 1 if we are to use a new session */
+	int quiet_shutdown;/* don't send shutdown packets */
 	int shutdown;	/* we have shut things down, 0x01 sent, 0x02
 			 * for received */
 	int state;	/* where we are */
@@ -498,12 +515,15 @@ typedef struct ssl_st
 
 	/* for server side, keep the list of CA_dn we can use */
 	STACK /* X509_NAME */ *client_CA;
+
+	int first_packet;
 	} SSL;
 
 #include "ssl2.h"
 #include "ssl3.h"
 #include "ssl23.h"
 
+/* application stuff */
 #define SSL_set_verify_result(s,arg)	((s)->verify_result=(long)arg)
 #define SSL_get_verify_result(s)	((s)->verify_result)
 #define SSL_set_app_data(s,arg)		((s)->app_data=(char *)arg)
@@ -546,6 +566,8 @@ typedef struct ssl_st
 #define SSL_CB_ACCEPT_EXIT		(SSL_ST_ACCEPT|SSL_CB_EXIT)
 #define SSL_CB_CONNECT_LOOP		(SSL_ST_CONNECT|SSL_CB_LOOP)
 #define SSL_CB_CONNECT_EXIT		(SSL_ST_CONNECT|SSL_CB_EXIT)
+#define SSL_CB_HANDSHAKE_START		0x10
+#define SSL_CB_HANDSHAKE_DONE		0x20
 
 /* Is the SSL_connection established? */
 #define SSL_is_init_finished(a)		((a)->state == SSL_ST_OK)
@@ -589,6 +611,15 @@ typedef struct ssl_st
 #define SSL_remove_session(a,b)		SSL_CTX_remove_session((a),(b))
 #define SSL_flush_sessions(a,b)		SSL_CTX_flush_sessions((a),(b))
 #endif
+/* More backward compatablity */
+#define SSL_get_cipher(s) \
+		SSL_CIPHER_get_name(SSL_get_current_cipher(s))
+#define SSL_get_cipher_bits(s,np) \
+		SSL_CIPHER_get_bits(SSL_get_current_cipher(s),np)
+#define SSL_get_cipher_version(s) \
+		SSL_CIPHER_get_version(SSL_get_current_cipher(s))
+#define SSL_get_cipher_name(s) \
+		SSL_CIPHER_get_name(SSL_get_current_cipher(s))
 
 /* VMS linker has a 31 char name limit */
 #define SSL_CTX_set_cert_verify_callback(a,b,c) \
@@ -649,7 +680,7 @@ typedef struct ssl_st
 BIO_METHOD *BIO_f_ssl(void);
 BIO *BIO_new_ssl(SSL_CTX *ctx,int client);
 int BIO_ssl_copy_session_id(BIO *to,BIO *from);
-int BIO_proxy_ssl_copy_session_id(BIO *t, BIO *f);
+void BIO_ssl_shutdown(BIO *ssl_bio);
 
 #endif
 
@@ -658,10 +689,12 @@ SSL_CTX *SSL_CTX_new(SSL_METHOD *meth);
 void	SSL_CTX_free(SSL_CTX *);
 void	SSL_clear(SSL *s);
 void	SSL_CTX_flush_sessions(SSL_CTX *ctx,long tm);
-char *	SSL_get_cipher(SSL *s);
-int	SSL_get_cipher_bits(SSL *s,int *alg_bits);
-char *	SSL_get_cipher_version(SSL_CIPHER *c);
-char *	SSL_get_cipher_name(SSL_CIPHER *c);
+
+SSL_CIPHER *SSL_get_current_cipher(SSL *s);
+int	SSL_CIPHER_get_bits(SSL_CIPHER *c,int *alg_bits);
+char *	SSL_CIPHER_get_version(SSL_CIPHER *c);
+char *	SSL_CIPHER_get_name(SSL_CIPHER *c);
+
 int	SSL_get_fd(SSL *s);
 char  * SSL_get_cipher_list(SSL *s,int n);
 char *	SSL_get_shared_ciphers(SSL *s, char *buf, int len);
@@ -813,17 +846,19 @@ X509 *SSL_get_certificate(SSL *ssl);
 BIO_METHOD *BIO_f_ssl();
 BIO *BIO_new_ssl();
 int BIO_ssl_copy_session_id();
-int BIO_proxy_ssl_copy_session_id();
+void BIO_ssl_shutdown();
 
 int	SSL_CTX_set_cipher_list();
 SSL_CTX *SSL_CTX_new();
 void	SSL_CTX_free();
 void	SSL_clear();
 void	SSL_CTX_flush_sessions();
-char *	SSL_get_cipher();
-int	SSL_get_cipher_bits();
-char *	SSL_get_cipher_version();
-char *	SSL_get_cipher_name();
+
+SSL_CIPHER *SSL_get_current_cipher();
+int	SSL_CIPHER_get_bits();
+char *	SSL_CIPHER_get_version();
+char *	SSL_CIPHER_get_name();
+
 int	SSL_get_fd();
 char  * SSL_get_cipher_list();
 char *	SSL_get_shared_ciphers();
@@ -1055,33 +1090,33 @@ EVP *SSL_get_privatekey();
 #define SSL_F_SSL_CTX_USE_RSAPRIVATEKEY			 174
 #define SSL_F_SSL_CTX_USE_RSAPRIVATEKEY_ASN1		 175
 #define SSL_F_SSL_CTX_USE_RSAPRIVATEKEY_FILE		 176
-#define SSL_F_SSL_GET_NEW_SESSION			 177
-#define SSL_F_SSL_GET_SERVER_SEND_CERT			 178
-#define SSL_F_SSL_GET_SIGN_PKEY				 179
-#define SSL_F_SSL_LOAD_CLIENT_CA_FILE			 180
-#define SSL_F_SSL_NEW					 181
-#define SSL_F_SSL_RSA_PRIVATE_DECRYPT			 182
-#define SSL_F_SSL_RSA_PUBLIC_ENCRYPT			 183
-#define SSL_F_SSL_SESSION_NEW				 184
-#define SSL_F_SSL_SESSION_PRINT_FP			 185
-#define SSL_F_SSL_SET_FD				 186
-#define SSL_F_SSL_SET_PKEY				 187
-#define SSL_F_SSL_SET_RFD				 188
-#define SSL_F_SSL_SET_SESSION				 189
-#define SSL_F_SSL_SET_WFD				 190
-#define SSL_F_SSL_UNDEFINED_FUNCTION			 191
-#define SSL_F_SSL_USE_CERTIFICATE			 192
-#define SSL_F_SSL_USE_CERTIFICATE_ASN1			 193
-#define SSL_F_SSL_USE_CERTIFICATE_FILE			 194
-#define SSL_F_SSL_USE_PRIVATEKEY			 195
-#define SSL_F_SSL_USE_PRIVATEKEY_ASN1			 196
-#define SSL_F_SSL_USE_PRIVATEKEY_FILE			 197
-#define SSL_F_SSL_USE_RSAPRIVATEKEY			 198
-#define SSL_F_SSL_USE_RSAPRIVATEKEY_ASN1		 199
-#define SSL_F_SSL_USE_RSAPRIVATEKEY_FILE		 200
-#define SSL_F_SSL_WRITE					 201
-#define SSL_F_WRITE_PENDING				 202
-#define SSL_F_SSL_DO_HANDSHAKE                           203
+#define SSL_F_SSL_DO_HANDSHAKE				 177
+#define SSL_F_SSL_GET_NEW_SESSION			 178
+#define SSL_F_SSL_GET_SERVER_SEND_CERT			 179
+#define SSL_F_SSL_GET_SIGN_PKEY				 180
+#define SSL_F_SSL_LOAD_CLIENT_CA_FILE			 181
+#define SSL_F_SSL_NEW					 182
+#define SSL_F_SSL_RSA_PRIVATE_DECRYPT			 183
+#define SSL_F_SSL_RSA_PUBLIC_ENCRYPT			 184
+#define SSL_F_SSL_SESSION_NEW				 185
+#define SSL_F_SSL_SESSION_PRINT_FP			 186
+#define SSL_F_SSL_SET_FD				 187
+#define SSL_F_SSL_SET_PKEY				 188
+#define SSL_F_SSL_SET_RFD				 189
+#define SSL_F_SSL_SET_SESSION				 190
+#define SSL_F_SSL_SET_WFD				 191
+#define SSL_F_SSL_UNDEFINED_FUNCTION			 192
+#define SSL_F_SSL_USE_CERTIFICATE			 193
+#define SSL_F_SSL_USE_CERTIFICATE_ASN1			 194
+#define SSL_F_SSL_USE_CERTIFICATE_FILE			 195
+#define SSL_F_SSL_USE_PRIVATEKEY			 196
+#define SSL_F_SSL_USE_PRIVATEKEY_ASN1			 197
+#define SSL_F_SSL_USE_PRIVATEKEY_FILE			 198
+#define SSL_F_SSL_USE_RSAPRIVATEKEY			 199
+#define SSL_F_SSL_USE_RSAPRIVATEKEY_ASN1		 200
+#define SSL_F_SSL_USE_RSAPRIVATEKEY_FILE		 201
+#define SSL_F_SSL_WRITE					 202
+#define SSL_F_WRITE_PENDING				 203
 
 /* Reason codes. */
 #define SSL_R_APP_DATA_IN_HANDSHAKE			 100
@@ -1128,74 +1163,75 @@ EVP *SSL_get_privatekey();
 #define SSL_R_CONNECTION_ID_IS_DIFFERENT		 141
 #define SSL_R_DATA_BETWEEN_CCS_AND_FINISHED		 142
 #define SSL_R_DATA_LENGTH_TOO_LONG			 143
-#define SSL_R_DIGEST_CHECK_FAILED			 144
-#define SSL_R_ENCRYPTED_LENGTH_TOO_LONG			 145
-#define SSL_R_ERROR_IN_RECEIVED_CIPHER_LIST		 146
-#define SSL_R_EXCESSIVE_MESSAGE_SIZE			 147
-#define SSL_R_EXTRA_DATA_IN_MESSAGE			 148
-#define SSL_R_GOT_A_FIN_BEFORE_A_CCS			 149
-#define SSL_R_INTERNAL_ERROR				 150
-#define SSL_R_INVALID_CHALLENGE_LENGTH			 151
-#define SSL_R_LENGTH_MISMATCH				 152
-#define SSL_R_LENGTH_TOO_SHORT				 153
-#define SSL_R_LIBRARY_HAS_NO_CIPHERS			 154
-#define SSL_R_MISSING_DH_DSA_CERT			 155
-#define SSL_R_MISSING_DH_KEY				 156
-#define SSL_R_MISSING_DH_RSA_CERT			 157
-#define SSL_R_MISSING_DSA_SIGNING_CERT			 158
-#define SSL_R_MISSING_EXPORT_TMP_DH_KEY			 159
-#define SSL_R_MISSING_EXPORT_TMP_RSA_KEY		 160
-#define SSL_R_MISSING_RSA_CERTIFICATE			 161
-#define SSL_R_MISSING_RSA_ENCRYPTING_CERT		 162
-#define SSL_R_MISSING_RSA_SIGNING_CERT			 163
-#define SSL_R_MISSING_TMP_DH_KEY			 164
-#define SSL_R_MISSING_TMP_RSA_KEY			 165
-#define SSL_R_MISSING_TMP_RSA_PKEY			 166
-#define SSL_R_MISSING_VERIFY_MESSAGE			 167
-#define SSL_R_NON_SSLV2_INITIAL_PACKET			 168
-#define SSL_R_NO_CERTIFICATES_PASSED			 169
-#define SSL_R_NO_CERTIFICATE_ASSIGNED			 170
-#define SSL_R_NO_CERTIFICATE_RETURNED			 171
-#define SSL_R_NO_CERTIFICATE_SET			 172
-#define SSL_R_NO_CERTIFICATE_SPECIFIED			 173
-#define SSL_R_NO_CIPHERS_AVAILABLE			 174
-#define SSL_R_NO_CIPHERS_PASSED				 175
-#define SSL_R_NO_CIPHERS_SPECIFIED			 176
-#define SSL_R_NO_CIPHER_LIST				 177
-#define SSL_R_NO_CIPHER_MATCH				 178
-#define SSL_R_NO_CLIENT_CERT_RECEIVED			 179
-#define SSL_R_NO_COMPRESSION_SPECIFIED			 180
-#define SSL_R_NO_PRIVATEKEY				 181
-#define SSL_R_NO_PRIVATE_KEY_ASSIGNED			 182
-#define SSL_R_NO_PUBLICKEY				 183
-#define SSL_R_NO_SHARED_CIPHER				 184
-#define SSL_R_NULL_SSL_CTX				 185
-#define SSL_R_NULL_SSL_METHOD_PASSED			 186
-#define SSL_R_OLD_SESSION_CIPHER_NOT_RETURNED		 187
-#define SSL_R_PACKET_LENGTH_TOO_LONG			 188
-#define SSL_R_PEER_DID_NOT_RETURN_A_CERTIFICATE		 189
-#define SSL_R_PEER_ERROR				 190
-#define SSL_R_PEER_ERROR_CERTIFICATE			 191
-#define SSL_R_PEER_ERROR_NO_CERTIFICATE			 192
-#define SSL_R_PEER_ERROR_NO_CIPHER			 193
-#define SSL_R_PEER_ERROR_UNSUPPORTED_CERTIFICATE_TYPE	 194
-#define SSL_R_PRE_MAC_LENGTH_TOO_LONG			 195
-#define SSL_R_PROBLEMS_MAPPING_CIPHER_FUNCTIONS		 196
-#define SSL_R_PROTOCOL_IS_SHUTDOWN			 197
-#define SSL_R_PUBLIC_KEY_ENCRYPT_ERROR			 198
-#define SSL_R_PUBLIC_KEY_IS_NOT_RSA			 199
-#define SSL_R_PUBLIC_KEY_NOT_RSA			 200
-#define SSL_R_READ_BIO_NOT_SET				 201
-#define SSL_R_READ_WRONG_PACKET_TYPE			 202
-#define SSL_R_RECORD_LENGTH_MISMATCH			 203
-#define SSL_R_RECORD_TOO_LARGE				 204
-#define SSL_R_REQUIRED_CIPHER_MISSING			 205
-#define SSL_R_REUSE_CERT_LENGTH_NOT_ZERO		 206
-#define SSL_R_REUSE_CERT_TYPE_NOT_ZERO			 207
-#define SSL_R_REUSE_CIPHER_LIST_NOT_ZERO		 208
-#define SSL_R_SHORT_READ				 209
-#define SSL_R_SIGNATURE_FOR_NON_SIGNING_CERTIFICATE	 210
-#define SSL_R_SSL3_SESSION_ID_TOO_SHORT			 211
+#define SSL_R_DH_PUBLIC_VALUE_LENGTH_IS_WRONG		 144
+#define SSL_R_DIGEST_CHECK_FAILED			 145
+#define SSL_R_ENCRYPTED_LENGTH_TOO_LONG			 146
+#define SSL_R_ERROR_IN_RECEIVED_CIPHER_LIST		 147
+#define SSL_R_EXCESSIVE_MESSAGE_SIZE			 148
+#define SSL_R_EXTRA_DATA_IN_MESSAGE			 149
+#define SSL_R_GOT_A_FIN_BEFORE_A_CCS			 150
+#define SSL_R_INTERNAL_ERROR				 151
+#define SSL_R_INVALID_CHALLENGE_LENGTH			 152
+#define SSL_R_LENGTH_MISMATCH				 153
+#define SSL_R_LENGTH_TOO_SHORT				 154
+#define SSL_R_LIBRARY_HAS_NO_CIPHERS			 155
+#define SSL_R_MISSING_DH_DSA_CERT			 156
+#define SSL_R_MISSING_DH_KEY				 157
+#define SSL_R_MISSING_DH_RSA_CERT			 158
+#define SSL_R_MISSING_DSA_SIGNING_CERT			 159
+#define SSL_R_MISSING_EXPORT_TMP_DH_KEY			 160
+#define SSL_R_MISSING_EXPORT_TMP_RSA_KEY		 161
+#define SSL_R_MISSING_RSA_CERTIFICATE			 162
+#define SSL_R_MISSING_RSA_ENCRYPTING_CERT		 163
+#define SSL_R_MISSING_RSA_SIGNING_CERT			 164
+#define SSL_R_MISSING_TMP_DH_KEY			 165
+#define SSL_R_MISSING_TMP_RSA_KEY			 166
+#define SSL_R_MISSING_TMP_RSA_PKEY			 167
+#define SSL_R_MISSING_VERIFY_MESSAGE			 168
+#define SSL_R_NON_SSLV2_INITIAL_PACKET			 169
+#define SSL_R_NO_CERTIFICATES_PASSED			 170
+#define SSL_R_NO_CERTIFICATE_ASSIGNED			 171
+#define SSL_R_NO_CERTIFICATE_RETURNED			 172
+#define SSL_R_NO_CERTIFICATE_SET			 173
+#define SSL_R_NO_CERTIFICATE_SPECIFIED			 174
+#define SSL_R_NO_CIPHERS_AVAILABLE			 175
+#define SSL_R_NO_CIPHERS_PASSED				 176
+#define SSL_R_NO_CIPHERS_SPECIFIED			 177
+#define SSL_R_NO_CIPHER_LIST				 178
+#define SSL_R_NO_CIPHER_MATCH				 179
+#define SSL_R_NO_CLIENT_CERT_RECEIVED			 180
+#define SSL_R_NO_COMPRESSION_SPECIFIED			 181
+#define SSL_R_NO_PRIVATEKEY				 182
+#define SSL_R_NO_PRIVATE_KEY_ASSIGNED			 183
+#define SSL_R_NO_PUBLICKEY				 184
+#define SSL_R_NO_SHARED_CIPHER				 185
+#define SSL_R_NULL_SSL_CTX				 186
+#define SSL_R_NULL_SSL_METHOD_PASSED			 187
+#define SSL_R_OLD_SESSION_CIPHER_NOT_RETURNED		 188
+#define SSL_R_PACKET_LENGTH_TOO_LONG			 189
+#define SSL_R_PEER_DID_NOT_RETURN_A_CERTIFICATE		 190
+#define SSL_R_PEER_ERROR				 191
+#define SSL_R_PEER_ERROR_CERTIFICATE			 192
+#define SSL_R_PEER_ERROR_NO_CERTIFICATE			 193
+#define SSL_R_PEER_ERROR_NO_CIPHER			 194
+#define SSL_R_PEER_ERROR_UNSUPPORTED_CERTIFICATE_TYPE	 195
+#define SSL_R_PRE_MAC_LENGTH_TOO_LONG			 196
+#define SSL_R_PROBLEMS_MAPPING_CIPHER_FUNCTIONS		 197
+#define SSL_R_PROTOCOL_IS_SHUTDOWN			 198
+#define SSL_R_PUBLIC_KEY_ENCRYPT_ERROR			 199
+#define SSL_R_PUBLIC_KEY_IS_NOT_RSA			 200
+#define SSL_R_PUBLIC_KEY_NOT_RSA			 201
+#define SSL_R_READ_BIO_NOT_SET				 202
+#define SSL_R_READ_WRONG_PACKET_TYPE			 203
+#define SSL_R_RECORD_LENGTH_MISMATCH			 204
+#define SSL_R_RECORD_TOO_LARGE				 205
+#define SSL_R_REQUIRED_CIPHER_MISSING			 206
+#define SSL_R_REUSE_CERT_LENGTH_NOT_ZERO		 207
+#define SSL_R_REUSE_CERT_TYPE_NOT_ZERO			 208
+#define SSL_R_REUSE_CIPHER_LIST_NOT_ZERO		 209
+#define SSL_R_SHORT_READ				 210
+#define SSL_R_SIGNATURE_FOR_NON_SIGNING_CERTIFICATE	 211
+#define SSL_R_SSL3_SESSION_ID_TOO_SHORT			 212
 #define SSL_R_SSLV3_ALERT_BAD_CERTIFICATE		 1042
 #define SSL_R_SSLV3_ALERT_BAD_RECORD_MAC		 1020
 #define SSL_R_SSLV3_ALERT_CERTIFICATE_EXPIRED		 1045
@@ -1205,47 +1241,47 @@ EVP *SSL_get_privatekey();
 #define SSL_R_SSLV3_ALERT_HANDSHAKE_FAILURE		 1040
 #define SSL_R_SSLV3_ALERT_ILLEGAL_PARAMETER		 1047
 #define SSL_R_SSLV3_ALERT_NO_CERTIFICATE		 1041
-#define SSL_R_SSLV3_ALERT_PEER_ERROR_CERTIFICATE	 212
-#define SSL_R_SSLV3_ALERT_PEER_ERROR_NO_CERTIFICATE	 213
-#define SSL_R_SSLV3_ALERT_PEER_ERROR_NO_CIPHER		 214
-#define SSL_R_SSLV3_ALERT_PEER_ERROR_UNSUPPORTED_CERTIFICATE_TYPE 215
+#define SSL_R_SSLV3_ALERT_PEER_ERROR_CERTIFICATE	 213
+#define SSL_R_SSLV3_ALERT_PEER_ERROR_NO_CERTIFICATE	 214
+#define SSL_R_SSLV3_ALERT_PEER_ERROR_NO_CIPHER		 215
+#define SSL_R_SSLV3_ALERT_PEER_ERROR_UNSUPPORTED_CERTIFICATE_TYPE 216
 #define SSL_R_SSLV3_ALERT_UNEXPECTED_MESSAGE		 1010
-#define SSL_R_SSLV3_ALERT_UNKNOWN_REMOTE_ERROR_TYPE	 216
+#define SSL_R_SSLV3_ALERT_UNKNOWN_REMOTE_ERROR_TYPE	 217
 #define SSL_R_SSLV3_ALERT_UNSUPPORTED_CERTIFICATE	 1043
-#define SSL_R_SSL_CTX_HAS_NO_DEFAULT_SSL_VERSION	 217
-#define SSL_R_SSL_HANDSHAKE_FAILURE			 218
-#define SSL_R_SSL_LIBRARY_HAS_NO_CIPHERS		 219
-#define SSL_R_SSL_SESSION_ID_IS_DIFFERENT		 220
-#define SSL_R_TRIED_TO_USE_UNSUPPORTED_CIPHER		 221
-#define SSL_R_UNABLE_TO_DECODE_DH_CERTS			 222
-#define SSL_R_UNABLE_TO_EXTRACT_PUBLIC_KEY		 223
-#define SSL_R_UNABLE_TO_FIND_DH_PARAMETERS		 224
-#define SSL_R_UNABLE_TO_FIND_PUBLIC_KEY_PARAMETERS	 225
-#define SSL_R_UNABLE_TO_FIND_SSL_METHOD			 226
-#define SSL_R_UNEXPECTED_MESSAGE			 227
-#define SSL_R_UNEXPECTED_RECORD				 228
-#define SSL_R_UNKNOWN_ALERT_TYPE			 229
-#define SSL_R_UNKNOWN_CERTIFICATE_TYPE			 230
-#define SSL_R_UNKNOWN_CIPHER_RETURNED			 231
-#define SSL_R_UNKNOWN_CIPHER_TYPE			 232
-#define SSL_R_UNKNOWN_KEY_EXCHANGE_TYPE			 233
-#define SSL_R_UNKNOWN_PKEY_TYPE				 234
-#define SSL_R_UNKNOWN_PROTOCOL				 235
-#define SSL_R_UNKNOWN_REMOTE_ERROR_TYPE			 236
-#define SSL_R_UNKNOWN_SSL_VERSION			 237
-#define SSL_R_UNKNOWN_STATE				 238
-#define SSL_R_UNSUPPORTED_CIPHER			 239
-#define SSL_R_UNSUPPORTED_COMPRESSION_ALGORITHM		 240
-#define SSL_R_UNSUPPORTED_SSL_VERSION			 241
-#define SSL_R_WRITE_BIO_NOT_SET				 242
-#define SSL_R_WRONG_CIPHER_RETURNED			 243
-#define SSL_R_WRONG_MESSAGE_TYPE			 244
-#define SSL_R_WRONG_NUMBER_OF_KEY_BITS			 245
-#define SSL_R_WRONG_SIGNATURE_LENGTH			 246
-#define SSL_R_WRONG_SIGNATURE_SIZE			 247
-#define SSL_R_WRONG_SSL_VERSION				 248
-#define SSL_R_WRONG_VERSION_NUMBER			 249
-#define SSL_R_X509_LIB					 250
+#define SSL_R_SSL_CTX_HAS_NO_DEFAULT_SSL_VERSION	 218
+#define SSL_R_SSL_HANDSHAKE_FAILURE			 219
+#define SSL_R_SSL_LIBRARY_HAS_NO_CIPHERS		 220
+#define SSL_R_SSL_SESSION_ID_IS_DIFFERENT		 221
+#define SSL_R_TRIED_TO_USE_UNSUPPORTED_CIPHER		 222
+#define SSL_R_UNABLE_TO_DECODE_DH_CERTS			 223
+#define SSL_R_UNABLE_TO_EXTRACT_PUBLIC_KEY		 224
+#define SSL_R_UNABLE_TO_FIND_DH_PARAMETERS		 225
+#define SSL_R_UNABLE_TO_FIND_PUBLIC_KEY_PARAMETERS	 226
+#define SSL_R_UNABLE_TO_FIND_SSL_METHOD			 227
+#define SSL_R_UNEXPECTED_MESSAGE			 228
+#define SSL_R_UNEXPECTED_RECORD				 229
+#define SSL_R_UNKNOWN_ALERT_TYPE			 230
+#define SSL_R_UNKNOWN_CERTIFICATE_TYPE			 231
+#define SSL_R_UNKNOWN_CIPHER_RETURNED			 232
+#define SSL_R_UNKNOWN_CIPHER_TYPE			 233
+#define SSL_R_UNKNOWN_KEY_EXCHANGE_TYPE			 234
+#define SSL_R_UNKNOWN_PKEY_TYPE				 235
+#define SSL_R_UNKNOWN_PROTOCOL				 236
+#define SSL_R_UNKNOWN_REMOTE_ERROR_TYPE			 237
+#define SSL_R_UNKNOWN_SSL_VERSION			 238
+#define SSL_R_UNKNOWN_STATE				 239
+#define SSL_R_UNSUPPORTED_CIPHER			 240
+#define SSL_R_UNSUPPORTED_COMPRESSION_ALGORITHM		 241
+#define SSL_R_UNSUPPORTED_SSL_VERSION			 242
+#define SSL_R_WRITE_BIO_NOT_SET				 243
+#define SSL_R_WRONG_CIPHER_RETURNED			 244
+#define SSL_R_WRONG_MESSAGE_TYPE			 245
+#define SSL_R_WRONG_NUMBER_OF_KEY_BITS			 246
+#define SSL_R_WRONG_SIGNATURE_LENGTH			 247
+#define SSL_R_WRONG_SIGNATURE_SIZE			 248
+#define SSL_R_WRONG_SSL_VERSION				 249
+#define SSL_R_WRONG_VERSION_NUMBER			 250
+#define SSL_R_X509_LIB					 251
  
 #ifdef  __cplusplus
 }
