@@ -67,7 +67,7 @@ static unsigned long conn_hash(SSL_SESSION *a);
 static unsigned long conn_hash();
 #endif
 
-char *SSL_version_str="SSLeay 0.8.0 25-Jun-1997";
+char *SSL_version_str="SSLeay 0.8.1a 24-Jun-1998";
 
 void SSL_clear(s)
 SSL *s;
@@ -84,6 +84,7 @@ SSL *s;
 	if (s->new_session) return;
 
 	state=s->state; /* Keep to check if we throw away the session-id */
+	s->type=0;
 
 	s->version=s->method->version;
 	s->rwstate=SSL_NOTHING;
@@ -108,6 +109,7 @@ SSL *s;
 		}
 
 	s->shutdown=(SSL_SENT_SHUTDOWN|SSL_RECEIVED_SHUTDOWN);
+	s->first_packet=0;
 
 	s->method->ssl_clear(s);
 	}
@@ -175,6 +177,7 @@ SSL_CTX *ctx;
 		goto err;
 		}
 
+	s->quiet_shutdown=ctx->quiet_shutdown;
 	SSL_clear(s);
 	return(s);
 err:
@@ -581,13 +584,25 @@ char *parg;
 int ssl_cipher_id_cmp(a,b)
 SSL_CIPHER *a,*b;
 	{
-	return((int)(a->id-b->id));
+	long l;
+
+	l=a->id-b->id;
+	if (l == 0L)
+		return(0);
+	else
+		return((l > 0)?1:-1);
 	}
 
 int ssl_cipher_ptr_id_cmp(ap,bp)
 SSL_CIPHER **ap,**bp;
 	{
-	return((int)((*ap)->id-(*bp)->id));
+	long l;
+
+	l=(*ap)->id-(*bp)->id;
+	if (l == 0L)
+		return(0);
+	else
+		return((l > 0)?1:-1);
 	}
 
 /* return a STACK of the ciphers available for the SSL and in order of
@@ -667,38 +682,6 @@ char *str;
 	return((sk == NULL)?0:1);
 	}
 
-/* return the actual cipher being used */
-char *SSL_get_cipher(s)
-SSL *s;
-	{
-	if ((s->session != NULL) && (s->session->cipher != NULL))
-		return(s->session->cipher->name);
-	return(NULL);
-	}
-
-/* number of bits for symetric cipher */
-int SSL_get_cipher_bits(s,alg_bits)
-SSL *s;
-int *alg_bits;
-	{
-	int ret=0,a=0;
-
-	if ((s->session != NULL) && (s->session->cipher != NULL) &&
-		(s->enc_read_ctx != NULL))
-		{
-		a=EVP_CIPHER_CTX_key_length(s->enc_read_ctx)*8;
-
-		if (s->session->cipher->algorithms & SSL_EXP)
-			ret=40;
-		else
-			ret=a;
-		}
-
-	if (alg_bits != NULL) *alg_bits=a;
-	
-	return(ret);
-	}
-
 /* works well for SSLv2, not so good for SSLv3 */
 char *SSL_get_shared_ciphers(s,buf,len)
 SSL *s;
@@ -865,6 +848,7 @@ SSL_METHOD *meth;
 	ret->sess_cb_hit=0;
 
 	ret->references=1;
+	ret->quiet_shutdown=0;
 
 /*	ret->cipher=NULL;*/
 /*	ret->s2->challenge=NULL;
@@ -1439,3 +1423,11 @@ SSL *s;
 		return(NULL);
 	}
 
+SSL_CIPHER *SSL_get_current_cipher(s)
+SSL *s;
+	{
+        if ((s->session != NULL) && (s->session->cipher != NULL))
+                return(s->session->cipher);
+        return(NULL);
+	}
+
