@@ -1,17 +1,22 @@
 /* ssl/ssl_sess.c */
-/* Copyright (C) 1995-1996 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1997 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
+ *
+ * This package is an SSL implementation written
+ * by Eric Young (eay@cryptsoft.com).
+ * The implementation was written so as to conform with Netscapes SSL.
  * 
- * This file is part of an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
- * The implementation was written so as to conform with Netscapes SSL
- * specification.  This library and applications are
- * FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
- * as long as the following conditions are aheared to.
+ * This library is free for commercial and non-commercial use as long as
+ * the following conditions are aheared to.  The following conditions
+ * apply to all code found in this distribution, be it the RC4, RSA,
+ * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
+ * included with this distribution is covered by the same copyright terms
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
- * the code are not to be removed.  If this code is used in a product,
- * Eric Young should be given attribution as the author of the parts used.
+ * the code are not to be removed.
+ * If this package is used in a product, Eric Young should be given attribution
+ * as the author of the parts of the library used.
  * This can be in the form of a textual message at program startup or
  * in documentation (online or textual) provided with the package.
  * 
@@ -25,7 +30,13 @@
  *    documentation and/or other materials provided with the distribution.
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
- *    This product includes software developed by Eric Young (eay@mincom.oz.au)
+ *    "This product includes cryptographic software written by
+ *     Eric Young (eay@cryptsoft.com)"
+ *    The word 'cryptographic' can be left out if the rouines from the library
+ *    being used are not cryptographic related :-).
+ * 4. If you include any Windows specific code (or a derivative thereof) from 
+ *    the apps directory (application code) you must include an acknowledgement:
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -46,11 +57,6 @@
  */
 
 #include <stdio.h>
-#ifdef NO_MD5
-#include <md5.h>
-#else
-#include "md5.h"
-#endif
 #include "lhash.h"
 #include "rand.h"
 #include "ssl_locl.h"
@@ -58,28 +64,17 @@
 SSL_SESSION *SSL_SESSION_new()
 	{
 	SSL_SESSION *ss;
-	int i;
 
-	ss=(SSL_SESSION *)malloc(sizeof(SSL_SESSION));
+	ss=(SSL_SESSION *)Malloc(sizeof(SSL_SESSION));
 	if (ss == NULL)
 		{
 		SSLerr(SSL_F_SSL_SESSION_NEW,ERR_R_MALLOC_FAILURE);
 		return(0);
 		}
-	ss->num_ciphers=0;
-	for (i=0; i<SSL_NUM_CIPHERS; i++) ss->ciphers[i]=NULL;
-	ss->cipher=NULL;
-	ss->key_arg_length=0;
-	ss->key_arg=NULL;
-	ss->master_key_length=0;
-	ss->master_key=NULL;
-	ss->session_id_length=0;
-	ss->session_id=NULL;
+	memset(ss,0,sizeof(SSL_SESSION));
 
-	ss->cert=NULL;
-	ss->peer=NULL;
 	ss->references=1;
-	ss->timeout=60*5; /* 5 minute timeout by default */
+	ss->timeout=60*5+4; /* 5 minute timeout by default */
 	ss->time=time(NULL);
 	return(ss);
 	}
@@ -94,7 +89,7 @@ int session;
 
 	/* If the context has a default timeout, use it */
 	if (s->ctx->session_timeout != 0)
-		ss->timeout=s->ctx->session_timeout;
+		ss->timeout=SSL_get_default_timeout(s);
 
 	if (s->session != NULL)
 		{
@@ -104,27 +99,35 @@ int session;
 
 	if (session)
 		{
-		ss->session_id_length=SSL_SESSION_ID_LENGTH;
-		ss->session_id=(unsigned char *)malloc(SSL_SESSION_ID_LENGTH);
-		if (ss->session_id == NULL)
+		if (s->version == SSL2_CLIENT_VERSION)
 			{
-			SSLerr(SSL_F_SSL_GET_NEW_SESSION,ERR_R_MALLOC_FAILURE);
+			ss->ssl_version=2;
+			ss->session_id_length=SSL2_SSL_SESSION_ID_LENGTH;
+			}
+		else if (s->version == SSL3_VERSION_MAJOR)
+			{
+			ss->ssl_version=3;
+			ss->session_id_length=SSL3_SSL_SESSION_ID_LENGTH;
+			}
+		else
+			{
+			SSLerr(SSL_F_SSL_GET_NEW_SESSION,SSL_R_UNSUPPORTED_SSL_VERSION);
 			SSL_SESSION_free(ss);
 			return(0);
 			}
 
-		CRYPTO_r_lock(CRYPTO_LOCK_SSL_CTX);
 		for (;;)
 			{
 			SSL_SESSION *r;
 
-			RAND_bytes(ss->session_id,SSL_SESSION_ID_LENGTH);
+			RAND_bytes(ss->session_id,ss->session_id_length);
+			CRYPTO_r_lock(CRYPTO_LOCK_SSL_CTX);
 			r=(SSL_SESSION *)lh_retrieve(s->ctx->sessions,
 				(char *)ss);
+			CRYPTO_r_unlock(CRYPTO_LOCK_SSL_CTX);
 			if (r == NULL) break;
 			/* else - woops a session_id match */
 			}
-		CRYPTO_r_unlock(CRYPTO_LOCK_SSL_CTX);
 		}
 	else
 		{
@@ -132,6 +135,8 @@ int session;
 		}
 
 	s->session=ss;
+	ss->ssl_version=s->version;
+
 	return(1);
 	}
 
@@ -143,8 +148,11 @@ unsigned char *session;
 	SSL_SESSION *ret,data;
 
 	/* conn_init();*/
+	data.ssl_version=s->version;
 	data.session_id_length=len;
-	data.session_id=session;
+	if (len > SSL_MAX_SSL_SESSION_ID_LENGTH)
+		return(0);
+	memcpy(data.session_id,session,len);;
 
 	CRYPTO_r_lock(CRYPTO_LOCK_SSL_CTX);
 	ret=(SSL_SESSION *)lh_retrieve(s->ctx->sessions,(char *)&data);
@@ -152,41 +160,58 @@ unsigned char *session;
 
 	if (ret == NULL)
 		{
-		CRYPTO_add(&s->ctx->sess_miss,1,CRYPTO_LOCK_SSL_CTX);
+		int copy=1;
+
+		s->ctx->sess_miss++;
 		ret=NULL;
 		if ((s->ctx->get_session_cb != NULL) &&
-			((ret=s->ctx->get_session_cb(session,len)) != NULL))
+			((ret=s->ctx->get_session_cb(s,session,len,&copy))
+				!= NULL))
 			{
-			CRYPTO_add(&s->ctx->sess_cb_hit,1,CRYPTO_LOCK_SSL_CTX);
+			s->ctx->sess_cb_hit++;
 
 			/* The following should not return 1, otherwise,
 			 * things are very strange */
-			SSL_add_session(s->ctx,ret);
+			SSL_CTX_add_session(s->ctx,ret);
+			/* auto free it */
+			if (!copy)
+				SSL_SESSION_free(ret);
 			}
 		if (ret == NULL) return(0);
 		}
-#ifdef SSL_DEBUG
-	SSL_TRACE(SSL_ERR,"GOT ONE\n");
-#endif
+
+	if (ret->cipher == NULL)
+		{
+		char buf[5],*p;
+		unsigned long l;
+
+		p=buf;
+		l=ret->cipher_id;
+		l2n(l,p);
+		if (ret->ssl_version == 3)
+			ret->cipher=ssl_get_cipher_by_char(s,&(buf[2]));
+		else 
+			ret->cipher=ssl_get_cipher_by_char(s,&(buf[1]));
+		if (ret->cipher == NULL)
+			return(0);
+		}
 
 	/* If a thread got the session, then 'swaped', and another got
-	 * it and then due to a time-out decided to 'free' it we could
+	 * it and then due to a time-out decided to 'Free' it we could
 	 * be in trouble.  So I'll increment it now, then double decrement
 	 * later - am I speaking rubbish?. */
 	CRYPTO_add(&ret->references,1,CRYPTO_LOCK_SSL_SESSION);
 
 	if ((long)(ret->time+ret->timeout) < (long)time(NULL)) /* timeout */
 		{
-#ifdef SSL_DEBUG
-		SSL_TRACE(SSL_ERR,"TIMEOUT ON SSL_SESSION\n");
-#endif
-		CRYPTO_add(&s->ctx->sess_timeout,1,CRYPTO_LOCK_SSL_CTX);
-		SSL_remove_session(s->ctx,ret); /* remove it from the cache */
-		SSL_SESSION_free(ret);		/* again to actually free it */
+		s->ctx->sess_timeout++;
+		/* remove it from the cache */
+		SSL_CTX_remove_session(s->ctx,ret);
+		SSL_SESSION_free(ret);		/* again to actually Free it */
 		return(0);
 		}
 
-	CRYPTO_add(&s->ctx->sess_hit,1,CRYPTO_LOCK_SSL_CTX);
+	s->ctx->sess_hit++;
 
 	/* ret->time=time(NULL); */ /* rezero timeout? */
 	/* again, just leave the session 
@@ -198,7 +223,7 @@ unsigned char *session;
 	return(1);
 	}
 
-int SSL_add_session(ctx,c)
+int SSL_CTX_add_session(ctx,c)
 SSL_CTX *ctx;
 SSL_SESSION *c;
 	{
@@ -211,7 +236,7 @@ SSL_SESSION *c;
 	s=(SSL_SESSION *)lh_insert(ctx->sessions,(char *)c);
 	CRYPTO_w_unlock(CRYPTO_LOCK_SSL_CTX);
 
-	/* If the same session if is being 're-added', free the old
+	/* If the same session if is being 're-added', Free the old
 	 * one when the last person stops using it.
 	 * This will also work if it is alread in the cache.
 	 * The references will go up and then down :-) */
@@ -224,27 +249,32 @@ SSL_SESSION *c;
 		return(1);
 	}
 
-void SSL_remove_session(ctx,c)
+int SSL_CTX_remove_session(ctx,c)
 SSL_CTX *ctx;
 SSL_SESSION *c;
 	{
 	SSL_SESSION *r;
+	int ret=0;
 
-	if ((c->session_id != NULL) && (c != NULL))
+	if ((c->session_id_length != 0) && (c != NULL))
 		{
 		CRYPTO_w_lock(CRYPTO_LOCK_SSL_CTX);
 		r=(SSL_SESSION *)lh_delete(ctx->sessions,(char *)c);
+		if (r != NULL) ret=1;
+
 		CRYPTO_w_unlock(CRYPTO_LOCK_SSL_CTX);
-		if (r == NULL)
+
+		if (ret)
 			{
-#ifdef SSL_DEBUG
-			SSL_TRACE(SSL_ERR,"freeing a session that is not hashed\n");
-#endif
-			return;
-			}
-		else
+			r->not_resumable=1;
+			if (ctx->remove_session_cb != NULL)
+				ctx->remove_session_cb(ctx,c);
 			SSL_SESSION_free(r);
+			}
 		}
+	else
+		ret=0;
+	return(ret);
 	}
 
 void SSL_SESSION_free(ss)
@@ -254,25 +284,22 @@ SSL_SESSION *ss;
 
 	i=CRYPTO_add(&ss->references,-1,CRYPTO_LOCK_SSL_SESSION);
 	if (i > 0) return;
+#ifdef REF_CHECK
+	if (i < 0)
+		{
+		fprintf(stderr,"SSL_SESSION_free, bad reference count\n");
+		abort(); /* ok */
+		}
+#endif
 
-	if (ss->key_arg != NULL)
-		{
-		memset(ss->key_arg,0,ss->key_arg_length);
-		free(ss->key_arg);
-		}
-	if (ss->master_key != NULL)
-		{
-		memset(ss->master_key,0,ss->master_key_length);
-		free(ss->master_key);
-		}
-	if (ss->session_id != NULL)
-		{
-		memset(ss->session_id,0,ss->session_id_length);
-		free(ss->session_id);
-		}
+	memset(ss->key_arg,0,SSL_MAX_KEY_ARG_LENGTH);
+	memset(ss->master_key,0,SSL_MAX_MASTER_KEY_LENGTH);
+	memset(ss->session_id,0,SSL_MAX_SSL_SESSION_ID_LENGTH);
 	if (ss->cert != NULL) ssl_cert_free(ss->cert);
 	if (ss->peer != NULL) X509_free(ss->peer);
-	free(ss);
+	if (ss->ciphers != NULL) sk_free(ss->ciphers);
+	memset(ss,0,sizeof(*ss));
+	Free(ss);
 	}
 
 int SSL_set_session(s, session)
@@ -280,13 +307,30 @@ SSL *s;
 SSL_SESSION *session;
 	{
 	int ret=0;
+	SSL_METHOD *meth;
 
 	if (session != NULL)
 		{
+		meth=s->ctx->method->get_ssl_method(session->ssl_version);
+		if (meth == NULL)
+			meth=s->method->get_ssl_method(session->ssl_version);
+		if (meth == NULL)
+			{
+			SSLerr(SSL_F_SSL_SET_SESSION,SSL_R_UNABLE_TO_FIND_SSL_METHOD);
+			return(0);
+			}
+
+		if (meth != s->method)
+			{
+			if (!SSL_set_ssl_method(s,meth))
+				return(0);
+			session->timeout=SSL_get_default_timeout(s);
+			}
+
 		/* CRYPTO_w_lock(CRYPTO_LOCK_SSL);*/
+		CRYPTO_add(&session->references,1,CRYPTO_LOCK_SSL_SESSION);
 		if (s->session != NULL)
 			SSL_SESSION_free(s->session);
-		CRYPTO_add(&session->references,1,CRYPTO_LOCK_SSL_SESSION);
 		s->session=session;
 		/* CRYPTO_w_unlock(CRYPTO_LOCK_SSL);*/
 		ret=1;
@@ -328,7 +372,8 @@ long t;
 
 typedef struct timeout_param_st
 	{
-	unsigned long time;
+	SSL_CTX *ctx;
+	long time;
 	LHASH *cache;
 	} TIMEOUT_PARAM;
 
@@ -339,17 +384,21 @@ TIMEOUT_PARAM *p;
 	if ((p->time == 0) || (p->time > (s->time+s->timeout))) /* timeout */
 		{
 		lh_delete(p->cache,(char *)s);
+		s->not_resumable=1;
+		if (p->ctx->remove_session_cb != NULL)
+			p->ctx->remove_session_cb(p->ctx,s);
 		SSL_SESSION_free(s);
 		}
 	}
 
-void SSL_flush_sessions(s,t)
+void SSL_CTX_flush_sessions(s,t)
 SSL_CTX *s;
 long t;
 	{
 	unsigned long i;
 	TIMEOUT_PARAM tp;
 
+	tp.ctx=s;
 	tp.cache=SSL_CTX_sessions(s);
 	if (tp.cache == NULL) return;
 	tp.time=t;
@@ -361,3 +410,16 @@ long t;
 	CRYPTO_w_unlock(CRYPTO_LOCK_SSL_CTX);
 	}
 
+int ssl_clear_bad_session(s)
+SSL *s;
+	{
+	if (	(s->session != NULL) &&
+		!(s->shutdown & SSL_SENT_SHUTDOWN) &&
+		!(SSL_in_init(s) || SSL_in_before(s)))
+		{
+		SSL_CTX_remove_session(s->ctx,s->session);
+		return(1);
+		}
+	else
+		return(0);
+	}
