@@ -320,6 +320,7 @@ SSL_CTX *s_ctx,*c_ctx;
 	int done=0;
 	int c_write,s_write;
 	int do_server=0,do_client=0;
+	SSL_CIPHER *ciph;
 
 	c_ssl=SSL_new(c_ctx);
 	s_ssl=SSL_new(s_ctx);
@@ -546,8 +547,9 @@ SSL_CTX *s_ctx,*c_ctx;
 		if ((done & S_DONE) && (done & C_DONE)) break;
 		}
 
-	fprintf(stdout,"DONE, used %s, %s\n",SSL_get_version(c_ssl),
-		SSL_get_cipher(c_ssl));
+	ciph=SSL_get_current_cipher(c_ssl);
+	fprintf(stdout,"DONE, used %s, %s\n",SSL_CIPHER_get_version(ciph),
+		SSL_CIPHER_get_name(ciph));
 	ret=0;
 err:
 	/* We have to set the BIO's to NULL otherwise they will be
