@@ -6,21 +6,21 @@
 # non-prototyped functions.
 #
 
-$crypto_num="util/crypto.num";
-$ssl_num=   "util/ssl.num";
+$crypto_num="util/libeay.num";
+$ssl_num=   "util/ssleay.num";
 
 $NT=1;
 foreach (@ARGV)
 	{
 	$NT=1 if $_ eq "32";
 	$NT=0 if $_ eq "16";
-	$do_ssl=1 if $_ eq "ssl";
-	$do_crypto=1 if $_ eq "crypto";
+	$do_ssl=1 if $_ eq "ssleay";
+	$do_crypto=1 if $_ eq "libeay";
 	}
 
 if (!$do_ssl && !$do_crypto)
 	{
-	print "usage: $0 ( ssl | crypto ) [ 16 | 32 ]\n";
+	print STDERR "usage: $0 ( ssl | crypto ) [ 16 | 32 ]\n";
 	exit(1);
 	}
 
@@ -34,8 +34,10 @@ $crypto.=" crypto/des/des.h";
 $crypto.=" crypto/idea/idea.h";
 $crypto.=" crypto/rc4/rc4.h";
 $crypto.=" crypto/rc2/rc2.h";
+$crypto.=" crypto/bf/blowfish.h";
 $crypto.=" crypto/md/md2.h";
 $crypto.=" crypto/md/md5.h";
+$crypto.=" crypto/mdc2/mdc2.h";
 $crypto.=" crypto/sha/sha.h";
 
 $crypto.=" crypto/bn/bn.h";
@@ -45,28 +47,30 @@ $crypto.=" crypto/dh/dh.h";
 
 $crypto.=" crypto/stack/stack.h";
 $crypto.=" crypto/buffer/buffer.h";
+$crypto.=" crypto/bio/bio.h";
 $crypto.=" crypto/lhash/lhash.h";
 $crypto.=" crypto/conf/conf.h";
 $crypto.=" crypto/txt_db/txt_db.h";
 
-$crypto.=" crypto/evp/envelope.h";
+$crypto.=" crypto/evp/evp.h";
 $crypto.=" crypto/objects/objects.h";
 $crypto.=" crypto/pem/pem.h";
-$crypto.=" crypto/meth/meth.h";
+#$crypto.=" crypto/meth/meth.h";
 $crypto.=" crypto/asn1/asn1.h";
 $crypto.=" crypto/asn1/asn1_mac.h";
-$crypto.=" crypto/error/err.h";
+$crypto.=" crypto/err/err.h";
 $crypto.=" crypto/pkcs7/pkcs7.h";
 $crypto.=" crypto/x509/x509.h";
+$crypto.=" crypto/x509/x509_vfy.h";
 $crypto.=" crypto/rand/rand.h";
 
 $match{'NOPROTO'}=1;
 $match2{'PERL5'}=1;
 
-&print_def_file(*STDOUT,"SSL",*ssl_list,&do_defs("SSL",$ssl))
+&print_def_file(*STDOUT,"SSLEAY",*ssl_list,&do_defs("SSLEAY",$ssl))
 	if $do_ssl == 1;
 
-&print_def_file(*STDOUT,"CRYPT",*crypto_list,&do_defs("CRYPT",$crypto))
+&print_def_file(*STDOUT,"LIBEAY",*crypto_list,&do_defs("LIBEAY",$crypto))
 	if $do_crypto == 1;
 
 sub do_defs
@@ -99,6 +103,12 @@ sub do_defs
 				$tag{$1}=-1;
 				next;
 				}
+			elsif (/^\#if !defined\(([^\)]+)\)/)
+				{
+				push(@tag,$1);
+				$tag{$1}=-1;
+				next;
+				}
 			elsif (/^\#ifdef (.*)/)
 				{
 				push(@tag,$1);
@@ -154,6 +164,10 @@ sub do_line
 		{ return($1); }
 	elsif (/(CRYPTO_get_id_callback)/)
 		{ return($1); }
+	elsif (/(CRYPTO_get_add_lock_callback)/)
+		{ return($1); }
+	elsif (/(SSL_CTX_get_verify_callback)/)
+		{ return($1); }
 	else
 		{
 		/\s\**(\S+)\s*\(/;
@@ -187,7 +201,7 @@ sub print_def_file
 
 LIBRARY         $name
 
-DESCRIPTION     'SSLeay $name - eay\@mincom.oz.au'
+DESCRIPTION     'SSLeay $name - eay\@cryptsoft.com'
 
 EOF
 
@@ -236,6 +250,8 @@ sub load_numbers
 	while (<IN>)
 		{
 		chop;
+		s/#.*$//;
+		next if /^\s*$/;
 		@a=split;
 		$ret{$a[0]}=$a[1];
 		}
